#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.romania import Romania
from tests.common import CommonCountryTests


class TestRomania(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Romania)

    def test_new_years_day(self):
        self.assertHolidayName(
            "Anul Nou",
            (f"{year}-01-01" for year in self.full_range),
            (f"{year}-01-02" for year in self.full_range),
        )

    def test_epiphany_day(self):
        name = "Botezul Domnului - Boboteaza"
        self.assertHolidayName(name, (f"{year}-01-06" for year in range(2024, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2024))

    def test_saint_john_baptist(self):
        name = "Soborul Sfântului Proroc Ioan Botezătorul"
        self.assertHolidayName(name, (f"{year}-01-07" for year in range(2024, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2024))

    def test_unification_of_romanian_principalities_day(self):
        name = "Ziua Unirii Principatelor Române"
        self.assertHolidayName(name, (f"{year}-01-24" for year in range(2017, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2017))

    def test_good_friday(self):
        name = "Vinerea Mare"
        self.assertHolidayName(
            name,
            "2020-04-17",
            "2021-04-30",
            "2022-04-22",
            "2023-04-14",
            "2024-05-03",
            "2025-04-18",
        )
        self.assertHolidayName(name, range(2018, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2018))

    def test_easter(self):
        name = "Paștele"
        self.assertHolidayName(
            name,
            "2020-04-19",
            "2020-04-20",
            "2021-05-02",
            "2021-05-03",
            "2022-04-24",
            "2022-04-25",
            "2023-04-16",
            "2023-04-17",
            "2024-05-05",
            "2024-05-06",
            "2025-04-20",
            "2025-04-21",
        )
        self.assertHolidayNameCount(name, 2, self.full_range)

    def test_labor_day(self):
        self.assertHolidayName("Ziua Muncii", (f"{year}-05-01" for year in self.full_range))

    def test_childrens_day(self):
        name = "Ziua Copilului"
        self.assertHolidayName(name, (f"{year}-06-01" for year in range(2017, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2017))

    def test_pentecost(self):
        name = "Rusaliile"
        self.assertHolidayName(
            name,
            "2020-06-07",
            "2020-06-08",
            "2021-06-20",
            "2021-06-21",
            "2022-06-12",
            "2022-06-13",
            "2023-06-04",
            "2023-06-05",
            "2024-06-23",
            "2024-06-24",
            "2025-06-08",
            "2025-06-09",
        )
        self.assertHolidayNameCount(name, 2, range(2009, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2009))

    def test_dormition_of_mother_of_god(self):
        name = "Adormirea Maicii Domnului"
        self.assertHolidayName(name, (f"{year}-08-15" for year in range(2009, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2009))

    def test_saint_andrews_day(self):
        name = "Sfântul Apostol Andrei, cel Întâi chemat, Ocrotitorul României"
        self.assertHolidayName(name, (f"{year}-11-30" for year in range(2012, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2012))

    def test_national_day(self):
        self.assertHolidayName(
            "Ziua Națională a României", (f"{year}-12-01" for year in self.full_range)
        )

    def test_christmas_day(self):
        self.assertHolidayName(
            "Crăciunul",
            (f"{year}-12-25" for year in self.full_range),
            (f"{year}-12-26" for year in self.full_range),
        )

    def test_2020(self):
        # https://publicholidays.ro/2020-dates/
        self.assertHolidayDatesInYear(
            2020,
            "2020-01-01",
            "2020-01-02",
            "2020-01-24",
            "2020-04-17",
            "2020-04-19",
            "2020-04-20",
            "2020-05-01",
            "2020-06-01",
            "2020-06-07",
            "2020-06-08",
            "2020-08-15",
            "2020-11-30",
            "2020-12-01",
            "2020-12-25",
            "2020-12-26",
        )

    def test_2022(self):
        # https://publicholidays.ro/2022-dates/
        self.assertHolidayDatesInYear(
            2022,
            "2022-01-01",
            "2022-01-02",
            "2022-01-24",
            "2022-04-22",
            "2022-04-24",
            "2022-04-25",
            "2022-05-01",
            "2022-06-01",
            "2022-06-12",
            "2022-06-13",
            "2022-08-15",
            "2022-11-30",
            "2022-12-01",
            "2022-12-25",
            "2022-12-26",
        )

    def test_2023(self):
        # https://publicholidays.ro/2023-dates/
        self.assertHolidayDatesInYear(
            2023,
            "2023-01-01",
            "2023-01-02",
            "2023-01-24",
            "2023-04-14",
            "2023-04-16",
            "2023-04-17",
            "2023-05-01",
            "2023-06-01",
            "2023-06-04",
            "2023-06-05",
            "2023-08-15",
            "2023-11-30",
            "2023-12-01",
            "2023-12-25",
            "2023-12-26",
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "Anul Nou"),
            ("2025-01-02", "Anul Nou"),
            ("2025-01-06", "Botezul Domnului - Boboteaza"),
            ("2025-01-07", "Soborul Sfântului Proroc Ioan Botezătorul"),
            ("2025-01-24", "Ziua Unirii Principatelor Române"),
            ("2025-04-18", "Vinerea Mare"),
            ("2025-04-20", "Paștele"),
            ("2025-04-21", "Paștele"),
            ("2025-05-01", "Ziua Muncii"),
            ("2025-06-01", "Ziua Copilului"),
            ("2025-06-08", "Rusaliile"),
            ("2025-06-09", "Rusaliile"),
            ("2025-08-15", "Adormirea Maicii Domnului"),
            ("2025-11-30", "Sfântul Apostol Andrei, cel Întâi chemat, Ocrotitorul României"),
            ("2025-12-01", "Ziua Națională a României"),
            ("2025-12-25", "Crăciunul"),
            ("2025-12-26", "Crăciunul"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-01-02", "New Year's Day"),
            ("2025-01-06", "Epiphany"),
            ("2025-01-07", "Saint John the Baptist"),
            ("2025-01-24", "Unification of the Romanian Principalities Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-20", "Easter"),
            ("2025-04-21", "Easter"),
            ("2025-05-01", "Labor Day"),
            ("2025-06-01", "Children's Day"),
            ("2025-06-08", "Pentecost"),
            ("2025-06-09", "Pentecost"),
            ("2025-08-15", "Dormition of the Mother of God"),
            ("2025-11-30", "Saint Andrew's Day"),
            ("2025-12-01", "National Day"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Christmas Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2025-01-01", "Новий рік"),
            ("2025-01-02", "Новий рік"),
            ("2025-01-06", "Богоявлення"),
            ("2025-01-07", "День Івана Хрестителя"),
            ("2025-01-24", "День обʼєднання Дунайських князівств"),
            ("2025-04-18", "Страсна пʼятниця"),
            ("2025-04-20", "Великдень"),
            ("2025-04-21", "Великдень"),
            ("2025-05-01", "День праці"),
            ("2025-06-01", "День захисту дітей"),
            ("2025-06-08", "Трійця"),
            ("2025-06-09", "Трійця"),
            ("2025-08-15", "Успіння Пресвятої Богородиці"),
            ("2025-11-30", "День Святого Андрія Первозваного, захисника Румунії"),
            ("2025-12-01", "Національний день Румунії"),
            ("2025-12-25", "Різдво Христове"),
            ("2025-12-26", "Різдво Христове"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import GOVERNMENT
from holidays.countries.macau import Macau
from tests.common import CommonCountryTests


class TestMacau(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Macau, with_subdiv_categories=True)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(Macau(years=range(self.start_year, 1985)))
        self.assertNoHolidays(Macau(categories=GOVERNMENT, years=range(self.start_year, 2005)))

    def test_special_holidays(self):
        dt_optional_full = (
            # https://www.io.gov.mo/pt/legis/rec/111020
            "1998-12-23",
            "1999-02-15",
            "1999-12-20",
            "1999-12-21",
            "2015-09-03",
        )
        dt_optional_half = (
            # https://www.io.gov.mo/pt/legis/rec/111020
            "1998-12-31",
            "1999-12-31",
            "2000-02-04",
        )
        dt_government_full = (
            # https://www.io.gov.mo/pt/legis/rec/111020
            "2008-12-22",
            # https://web.archive.org/web/20171207162948/http://portal.gov.mo/web/guest/info_detail?infoid=1887061
            "2012-10-03",
            "2012-12-31",
            "2014-10-03",
            # https://www.gov.mo/en/public-holidays/year-2020/
            "2020-10-05",
        )
        dt_mandatory = (
            # https://www.dsal.gov.mo/en/text/holiday_table.html
            "2015-09-03"
        )
        self.assertHoliday(dt_mandatory)
        self.assertGovernmentHoliday(dt_government_full)
        self.assertOptionalHoliday(dt_optional_full, dt_optional_half)

    def test_new_years_day(self):
        name = "元旦"

        # Government Holidays.
        self.assertGovernmentHolidayName(
            name, (f"{year}-01-01" for year in range(2005, self.end_year))
        )
        obs_dts_1 = (
            "2012-01-02",
            "2017-01-02",
        )
        obs_dts_2 = (
            "2022-01-03",
            "2023-01-02",
            "2028-01-03",
        )
        self.assertGovernmentHolidayName(f"{name}後首個工作日", obs_dts_1)
        self.assertGovernmentHolidayName(f"{name}的補假", obs_dts_2)
        self.assertNoGovernmentNonObservedHoliday(obs_dts_1, obs_dts_2)

        # Optional Holidays.
        self.assertOptionalHolidayName(name, (f"{year}-01-01" for year in self.full_range))

        # Public Holidays.
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(1985, self.end_year)))

    def test_freedom_day(self):
        name = "自由日"

        # Government Holidays.
        self.assertNoGovernmentHolidayName(name)

        # Optional Holidays.
        self.assertOptionalHolidayName(
            name, (f"{year}-04-25" for year in range(self.start_year, 2000))
        )
        self.assertNoOptionalHolidayName(name, range(2000, self.end_year))

        # Public Holidays.
        self.assertNoHolidayName(name)

    def test_labor_day(self):
        name = "勞動節"

        # Government Holidays.
        self.assertGovernmentHolidayName(
            name, (f"{year}-05-01" for year in range(2005, self.end_year))
        )
        obs_dts_1 = (
            "2011-05-02",
            "2016-05-02",
        )
        obs_dts_2 = (
            "2021-05-03",
            "2022-05-02",
            "2027-05-03",
        )
        self.assertGovernmentHolidayName(f"{name}後首個工作日", obs_dts_1)
        self.assertGovernmentHolidayName(f"{name}的補假", obs_dts_2)
        self.assertNoGovernmentNonObservedHoliday(obs_dts_1, obs_dts_2)

        # Optional Holidays.
        self.assertOptionalHolidayName(name, (f"{year}-05-01" for year in self.full_range))

        # Public Holidays.
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1985, self.end_year)))

    def test_day_of_portugal_camoes_and_the_portuguese_communities(self):
        name = "葡國日、賈梅士日暨葡僑日"

        # Government Holidays.
        self.assertNoGovernmentHolidayName(name)

        # Optional Holidays.
        self.assertOptionalHolidayName(
            name, (f"{year}-06-10" for year in range(self.start_year, 2000))
        )
        self.assertNoOptionalHolidayName(name, range(2000, self.end_year))

        # Public Holidays.
        self.assertHolidayName(name, (f"{year}-06-10" for year in range(1985, 2000)))
        self.assertNoHolidayName(name, range(2000, self.end_year))

    def test_assumption_day(self):
        name = "聖母升天"

        # Government Holidays.
        self.assertNoGovernmentHolidayName(name)

        # Optional Holidays.
        self.assertOptionalHolidayName(
            name, (f"{year}-08-15" for year in range(self.start_year, 1987))
        )
        self.assertNoOptionalHolidayName(name, range(1987, self.end_year))

        # Public Holidays.
        self.assertNoHolidayName(name)

    def test_national_day(self):
        name = "中華人民共和國國慶日"
        name_following = f"{name}翌日"

        # Government Holidays.
        self.assertGovernmentHolidayName(
            name, (f"{year}-10-01" for year in range(2005, self.end_year))
        )
        obs_dts_1 = (
            "2016-10-03",
            "2017-10-03",
        )
        obs_dts_2 = (
            "2022-10-03",
            "2023-10-04",
        )
        obs_dts_following_1 = (
            "2011-10-03",
            "2016-10-04",
        )
        obs_dts_following_2 = (
            "2021-10-04",
            "2022-10-05",
        )
        self.assertGovernmentHolidayName(f"{name}後首個工作日", obs_dts_1)
        self.assertGovernmentHolidayName(f"{name}的補假", obs_dts_2)
        self.assertGovernmentHolidayName(f"{name_following}後首個工作日", obs_dts_following_1)
        self.assertGovernmentHolidayName(f"{name_following}的補假", obs_dts_following_2)
        self.assertNoGovernmentNonObservedHoliday(
            obs_dts_1, obs_dts_2, obs_dts_following_1, obs_dts_following_2
        )

        # Optional Holidays.
        self.assertOptionalHolidayName(name, (f"{year}-10-01" for year in self.full_range))
        self.assertOptionalHolidayName(
            name_following, (f"{year}-10-02" for year in range(2000, self.end_year))
        )
        self.assertNoOptionalHolidayName(name_following, range(self.start_year, 2000))

        # Public Holidays.
        self.assertHolidayName(name, (f"{year}-10-01" for year in range(1985, self.end_year)))

    def test_republic_day(self):
        name = "葡萄牙共和國國慶日"

        # Government Holidays.
        self.assertNoGovernmentHolidayName(name)

        # Optional Holidays.
        self.assertOptionalHolidayName(
            name, (f"{year}-10-05" for year in range(self.start_year, 2000))
        )
        self.assertNoOptionalHolidayName(name, range(2000, self.end_year))

        # Public Holidays.
        self.assertNoHolidayName(name)

    def test_all_saints_day(self):
        name = "諸聖節"

        # Government Holidays.
        self.assertNoGovernmentHolidayName(name)

        # Optional Holidays.
        self.assertOptionalHolidayName(
            name, (f"{year}-11-01" for year in range(self.start_year, 1987))
        )
        self.assertNoOptionalHolidayName(name, range(1987, self.end_year))

        # Public Holidays.
        self.assertNoHolidayName(name)

    def test_all_souls_day(self):
        name = "追思節"

        # Government Holidays.
        self.assertGovernmentHolidayName(
            name, (f"{year}-11-02" for year in range(2005, self.end_year))
        )
        obs_dts_1 = (
            "2014-11-03",
            "2019-11-04",
        )
        obs_dts_2 = (
            "2024-11-04",
            "2025-11-03",
        )
        self.assertGovernmentHolidayName(f"{name}後首個工作日", obs_dts_1)
        self.assertGovernmentHolidayName(f"{name}的補假", obs_dts_2)
        self.assertNoGovernmentNonObservedHoliday(obs_dts_1, obs_dts_2)

        # Optional Holidays.
        self.assertOptionalHolidayName(name, (f"{year}-11-02" for year in self.full_range))

        # Public Holidays.
        self.assertNoHolidayName(name)

    def test_restoration_of_independence_day(self):
        name = "恢復獨立紀念日"

        # Government Holidays.
        self.assertNoGovernmentHolidayName(name)

        # Optional Holidays.
        self.assertOptionalHolidayName(
            name, (f"{year}-12-01" for year in range(self.start_year, 2000))
        )
        self.assertNoOptionalHolidayName(name, range(2000, self.end_year))

        # Public Holidays.
        self.assertNoHolidayName(name)

    def test_immaculate_conception(self):
        name = "聖母無原罪瞻禮"

        # Government Holidays.
        self.assertGovernmentHolidayName(
            name, (f"{year}-12-08" for year in range(2005, self.end_year))
        )
        obs_dts_1 = (
            "2012-12-10",
            "2013-12-09",
            "2018-12-10",
            "2019-12-09",
        )
        obs_dts_2 = ("2024-12-09",)
        self.assertGovernmentHolidayName(f"{name}後首個工作日", obs_dts_1)
        self.assertGovernmentHolidayName(f"{name}的補假", obs_dts_2)
        self.assertNoGovernmentNonObservedHoliday(obs_dts_1, obs_dts_2)

        # Optional Holidays.
        self.assertOptionalHolidayName(name, (f"{year}-12-08" for year in self.full_range))

        # Public Holidays.
        self.assertNoHolidayName(name)

    def test_macao_sar_establishment_day(self):
        name = "澳門特別行政區成立紀念日"

        # Government Holidays.
        self.assertGovernmentHolidayName(
            name, (f"{year}-12-20" for year in range(2005, self.end_year))
        )
        obs_dts_1 = (
            "2014-12-23",
            "2015-12-21",
        )
        obs_dts_2 = (
            "2020-12-22",
            "2025-12-22",
        )
        self.assertGovernmentHolidayName(f"{name}後首個工作日", obs_dts_1)
        self.assertGovernmentHolidayName(f"{name}的補假", obs_dts_2)
        self.assertNoGovernmentNonObservedHoliday(obs_dts_1, obs_dts_2)

        # Optional Holidays.
        self.assertOptionalHolidayName(
            name, (f"{year}-12-20" for year in range(2000, self.end_year))
        )
        self.assertNoOptionalHolidayName(name, range(self.start_year, 2000))

        # Public Holidays.
        self.assertHolidayName(name, (f"{year}-12-20" for year in range(2000, self.end_year)))
        self.assertNoHolidayName(name, range(1985, 2000))

    def test_winter_solstice(self):
        name = "冬至"
        dts = (
            "2018-12-22",
            "2019-12-22",
            "2020-12-21",
            "2021-12-21",
            "2022-12-22",
            "2023-12-22",
            "2024-12-21",
            "2025-12-21",
        )

        # Government Holidays.
        self.assertGovernmentHolidayName(name, dts)
        self.assertGovernmentHolidayName(name, range(2005, self.end_year))
        obs_dts_1 = (
            "2013-12-23",
            "2018-12-26",
            "2019-12-23",
        )
        obs_dts_2 = (
            "2024-12-23",
            "2025-12-23",
        )
        self.assertGovernmentHolidayName(f"{name}後首個工作日", obs_dts_1)
        self.assertGovernmentHolidayName(f"{name}的補假", obs_dts_2)
        self.assertNoGovernmentNonObservedHoliday(obs_dts_1, obs_dts_2)

        # Optional Holidays.
        self.assertOptionalHolidayName(
            name, (f"{year}-12-22" for year in range(self.start_year, 2000))
        )
        self.assertOptionalHolidayName(
            name,
            # Regulamento Administrativo n.º 4/1999.
            "2000-12-21",
            # Ordem Executiva n.º 60/2000.
            dts,
        )
        self.assertOptionalHolidayName(name, range(2000, self.end_year))

        # Public Holidays.
        self.assertNoHolidayName(name)

    def test_christmas_eve(self):
        name_1982 = "聖誕前夕"
        name_1999 = "聖誕節前夕"
        name_2000 = "聖誕節前日"

        # Government Holidays.
        self.assertGovernmentHolidayName(
            name_2000, (f"{year}-12-24" for year in range(2005, self.end_year))
        )
        obs_dts_1 = (
            "2016-12-26",
            "2017-12-26",
        )
        obs_dts_2 = (
            "2022-12-26",
            "2023-12-26",
        )
        self.assertGovernmentHolidayName(f"{name_2000}後首個工作日", obs_dts_1)
        self.assertGovernmentHolidayName(f"{name_2000}的補假", obs_dts_2)
        self.assertNoGovernmentNonObservedHoliday(obs_dts_1, obs_dts_2)

        # Optional Holidays.
        self.assertOptionalHolidayName(
            name_1982, (f"{year}-12-24" for year in range(self.start_year, 1999))
        )
        self.assertOptionalHolidayName(name_1999, "1999-12-24")
        self.assertOptionalHolidayName(
            name_2000, (f"{year}-12-24" for year in range(2000, self.end_year))
        )
        self.assertNoOptionalHolidayName(name_1982, range(1999, self.end_year))
        self.assertNoOptionalHolidayName(
            name_1999, range(self.start_year, 1999), range(2000, self.end_year)
        )
        self.assertNoOptionalHolidayName(name_2000, range(self.start_year, 2000))

        # Public Holidays.
        self.assertNoHolidayName(name_1982)
        self.assertNoHolidayName(name_1999)
        self.assertNoHolidayName(name_2000)

    def test_christmas_day(self):
        name_1982 = "聖誕"
        name_1999 = "聖誕節"

        # Government Holidays.
        self.assertGovernmentHolidayName(
            name_1999, (f"{year}-12-25" for year in range(2005, self.end_year))
        )
        obs_dts_1 = (
            "2011-12-26",
            "2016-12-27",
        )
        obs_dts_2 = (
            "2021-12-27",
            "2022-12-27",
        )
        self.assertGovernmentHolidayName(f"{name_1999}後首個工作日", obs_dts_1)
        self.assertGovernmentHolidayName(f"{name_1999}的補假", obs_dts_2)
        self.assertNoGovernmentNonObservedHoliday(obs_dts_1, obs_dts_2)

        # Optional Holidays.
        self.assertOptionalHolidayName(
            name_1982, (f"{year}-12-25" for year in range(self.start_year, 1999))
        )
        self.assertOptionalHolidayName(
            name_1999, (f"{year}-12-25" for year in range(1999, self.end_year))
        )
        self.assertNoOptionalHolidayName(name_1982, range(1999, self.end_year))
        self.assertNoOptionalHolidayName(name_1999, range(self.start_year, 1999))

        # Public Holidays.
        self.assertNoHolidayName(name_1982)
        self.assertNoHolidayName(name_1999)

    def test_chinese_new_year(self):
        name_eve_afternoon = "農曆除夕（下午）"
        name_d1 = "農曆正月初一"
        name_d2 = "農曆正月初二"
        name_d3 = "農曆正月初三"
        name_d4 = "農曆正月初四"
        name_d5 = "農曆正月初五"
        name_d1_obs = f"{name_d1}的補假"
        name_d2_obs = f"{name_d2}的補假"
        name_d3_obs = f"{name_d3}的補假"

        dts = (
            "2020-01-25",
            "2021-02-12",
            "2022-02-01",
            "2024-02-10",
            "2025-01-29",
        )

        # Government Holidays.
        self.assertGovernmentHolidayName(
            name_eve_afternoon,
            "2005-02-08",
            "2008-02-06",
            "2011-02-02",
            "2014-01-30",
            "2015-02-18",
            "2017-01-27",
            "2018-02-15",
            "2019-02-04",
            "2020-01-24",
            "2021-02-11",
            "2022-01-31",
            "2024-02-09",
            "2025-01-28",
        )
        self.assertNoGovernmentHolidayName(
            name_eve_afternoon, 2006, 2007, 2009, 2010, 2012, 2013, 2016, 2023
        )

        self.assertGovernmentHolidayName(name_d1, dts)
        self.assertGovernmentHolidayName(name_d1, range(2019, self.end_year))
        self.assertGovernmentHolidayName(name_d2, range(2019, self.end_year))
        self.assertGovernmentHolidayName(name_d3, range(2019, self.end_year))
        self.assertNoGovernmentHolidayName(name_d1, range(self.start_year, 2019))
        self.assertNoGovernmentHolidayName(name_d2, range(self.start_year, 2019))
        self.assertNoGovernmentHolidayName(name_d3, range(self.start_year, 2019))

        self.assertGovernmentHolidayName(
            name_d4,
            "2006-02-01",
            "2007-02-21",
            "2010-02-17",
            "2013-02-13",
            "2014-02-03",
            "2017-01-31",
            "2018-02-19",
        )
        self.assertNoGovernmentHolidayName(
            name_d4,
            2005,
            2008,
            2009,
            2011,
            2012,
            2015,
            2016,
            range(2019, self.end_year),
        )

        self.assertGovernmentHolidayName(
            name_d5,
            "2014-02-04",
            "2015-02-23",
            "2017-02-01",
            "2018-02-20",
        )
        self.assertNoGovernmentHolidayName(
            name_d5, range(2005, 2014), 2016, range(2019, self.end_year)
        )

        obs_dts_1 = (
            "2020-01-28",
            "2023-01-25",
            "2024-02-13",
        )
        obs_dts_2 = (
            "2020-01-29",
            "2021-02-15",
            "2024-02-14",
        )
        obs_dts_3 = ("2021-02-16",)
        self.assertGovernmentHolidayName(name_d1_obs, obs_dts_1)
        self.assertGovernmentHolidayName(name_d2_obs, obs_dts_2)
        self.assertGovernmentHolidayName(name_d3_obs, obs_dts_3)
        self.assertNoGovernmentNonObservedHoliday(obs_dts_1, obs_dts_2, obs_dts_3)

        # Optional Holidays.
        self.assertOptionalHolidayName(name_d1, dts)
        self.assertOptionalHolidayName(name_d1, self.full_range)
        self.assertOptionalHolidayName(name_d2, self.full_range)
        self.assertOptionalHolidayName(name_d3, self.full_range)

        # Public Holidays.
        self.assertHolidayName(name_d1, dts)
        self.assertHolidayName(name_d1, range(1985, self.end_year))
        self.assertHolidayName(name_d2, range(1985, self.end_year))
        self.assertHolidayName(name_d3, range(1985, self.end_year))
        self.assertNoHolidayName(name_eve_afternoon)
        self.assertNoHolidayName(name_d4)
        self.assertNoHolidayName(name_d5)

    def test_tomb_sweeping_day(self):
        name = "清明節"
        dts = (
            "2020-04-04",
            "2022-04-05",
            "2023-04-05",
            "2024-04-04",
            "2025-04-04",
        )

        # Government Holidays.
        self.assertGovernmentHolidayName(name, dts)
        self.assertGovernmentHolidayName(name, range(2005, self.end_year))
        obs_dts_1 = (
            "2014-04-07",
            "2015-04-07",
        )
        obs_dts_2 = (
            "2020-04-06",
            "2021-04-06",
        )
        self.assertGovernmentHolidayName(f"{name}後首個工作日", obs_dts_1)
        self.assertGovernmentHolidayName(f"{name}的補假", obs_dts_2)
        self.assertNoGovernmentNonObservedHoliday(obs_dts_1, obs_dts_2)

        # Optional Holidays.
        self.assertOptionalHolidayName(name, dts)
        self.assertOptionalHolidayName(name, self.full_range)

        # Public Holidays.
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, range(1989, self.end_year))
        self.assertNoHolidayName(name, range(1985, 1989))

    def test_good_friday(self):
        name_1982 = "聖周星期五"
        name_2000 = "耶穌受難日"

        # Government Holidays.
        self.assertNoGovernmentHolidayName(name_1982)
        self.assertNoGovernmentHolidayName(name_2000)

        # Optional Holidays.
        self.assertOptionalHolidayName(
            name_2000,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertOptionalHolidayName(name_1982, range(self.start_year, 2000))
        self.assertOptionalHolidayName(name_2000, range(2000, self.end_year))
        self.assertNoOptionalHolidayName(name_1982, range(2000, self.end_year))
        self.assertNoOptionalHolidayName(name_2000, range(self.start_year, 2000))

        # Public Holidays.
        self.assertNoHolidayName(name_1982)
        self.assertNoHolidayName(name_2000)

    def test_the_day_before_easter(self):
        name_1982 = "聖周星期六"
        name_2000 = "復活節前日"
        dts = (
            "2020-04-11",
            "2021-04-03",
            "2022-04-16",
            "2023-04-08",
            "2024-03-30",
            "2025-04-19",
        )

        # Government Holidays.
        name_2000_obs_2005 = f"{name_2000}後首個工作日"
        name_2000_obs_2020 = f"{name_2000}的補假"
        self.assertGovernmentHolidayName(name_2000, dts)
        self.assertGovernmentHolidayName(name_2000, range(2005, self.end_year))
        obs_dts_1 = (
            "2015-04-06",
            "2016-03-28",
            "2017-04-17",
            "2018-04-02",
            "2019-04-22",
        )
        obs_dts_2 = (
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertGovernmentHolidayName(name_2000_obs_2005, obs_dts_1)
        self.assertGovernmentHolidayName(name_2000_obs_2005, range(2012, 2020))
        self.assertGovernmentHolidayName(name_2000_obs_2020, obs_dts_2)
        self.assertGovernmentHolidayName(name_2000_obs_2020, range(2020, self.end_year))
        self.assertNoGovernmentHolidayName(
            name_2000_obs_2005, range(2005, 2012), range(2020, self.end_year)
        )
        self.assertNoGovernmentHolidayName(name_2000_obs_2020, range(self.start_year, 2020))
        self.assertNoGovernmentNonObservedHoliday(obs_dts_1, obs_dts_2)

        # Optional Holidays.
        self.assertOptionalHolidayName(name_2000, dts)
        self.assertOptionalHolidayName(name_1982, range(self.start_year, 2000))
        self.assertOptionalHolidayName(name_2000, range(2000, self.end_year))
        self.assertNoOptionalHolidayName(name_1982, range(2000, self.end_year))
        self.assertNoOptionalHolidayName(name_2000, range(self.start_year, 2000))

        # Public Holidays.
        self.assertNoHolidayName(name_1982)
        self.assertNoHolidayName(name_2000)

    def test_the_buddhas_birthday(self):
        name = "佛誕節"
        dts = (
            "2020-04-30",
            "2021-05-19",
            "2022-05-08",
            "2023-05-26",
            "2024-05-15",
            "2025-05-05",
        )

        # Government Holidays.
        self.assertGovernmentHolidayName(name, dts)
        self.assertGovernmentHolidayName(name, range(2005, self.end_year))
        obs_dts_1 = (
            "2012-04-30",
            "2016-05-16",
            "2019-05-13",
        )
        obs_dts_2 = ("2022-05-09",)
        self.assertGovernmentHolidayName(f"{name}後首個工作日", obs_dts_1)
        self.assertGovernmentHolidayName(f"{name}的補假", obs_dts_2)
        self.assertNoGovernmentNonObservedHoliday(obs_dts_1, obs_dts_2)

        # Optional Holidays.
        self.assertOptionalHolidayName(name, dts)
        self.assertOptionalHolidayName(name, range(2000, self.end_year))
        self.assertNoOptionalHolidayName(name, range(self.start_year, 2000))

        # Public Holidays.
        self.assertNoHolidayName(name)

    def test_corpus_christi(self):
        name = "基督聖體聖血節"

        # Government Holidays.
        self.assertNoGovernmentHolidayName(name)

        # Optional Holidays.
        self.assertOptionalHolidayName(
            name,
            "1982-06-10",
            "1983-06-02",
            "1984-06-21",
            "1985-06-06",
            "1986-05-29",
            "1987-06-18",
        )
        self.assertNoOptionalHolidayName(name, range(1988, self.end_year))

        # Public Holidays.
        self.assertNoHolidayName(name)

    def test_dragon_boat_festival(self):
        name = "端午節"
        dts = (
            "2020-06-25",
            "2021-06-14",
            "2022-06-03",
            "2023-06-22",
            "2024-06-10",
            "2025-05-31",
        )

        # Government Holidays.
        self.assertGovernmentHolidayName(name, dts)
        self.assertGovernmentHolidayName(name, range(2005, self.end_year))
        obs_dts_1 = (
            "2012-06-25",
            "2015-06-22",
        )
        obs_dts_2 = ("2025-06-02",)
        self.assertGovernmentHolidayName(f"{name}後首個工作日", obs_dts_1)
        self.assertGovernmentHolidayName(f"{name}的補假", obs_dts_2)
        self.assertNoGovernmentNonObservedHoliday(obs_dts_1, obs_dts_2)

        # Optional Holidays.
        self.assertOptionalHolidayName(name, dts)
        self.assertOptionalHolidayName(name, self.full_range)

        # Public Holidays.
        self.assertNoHolidayName(name)

    def test_double_ninth_festival(self):
        name = "重陽節"
        dts = (
            "2020-10-25",
            "2021-10-14",
            "2022-10-04",
            "2023-10-23",
            "2024-10-11",
            "2025-10-29",
        )

        # Government Holidays.
        self.assertGovernmentHolidayName(name, dts)
        self.assertGovernmentHolidayName(name, range(2005, self.end_year))
        obs_dts_1 = (
            "2013-10-14",
            "2016-10-10",
            "2017-10-30",
        )
        obs_dts_2 = ("2020-10-26",)
        self.assertGovernmentHolidayName(f"{name}後首個工作日", obs_dts_1)
        self.assertGovernmentHolidayName(f"{name}的補假", obs_dts_2)
        self.assertNoGovernmentNonObservedHoliday(obs_dts_1, obs_dts_2)

        # Optional Holidays.
        self.assertOptionalHolidayName(name, dts)
        self.assertOptionalHolidayName(name, self.full_range)

        # Public Holidays.
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, range(1985, self.end_year))

    def test_mid_autumn_festival(self):
        name = "中秋節"

        # Government Holidays.
        self.assertNoGovernmentHolidayName(name)

        # Optional Holidays.
        self.assertNoOptionalHolidayName(name)

        # Public Holidays.
        self.assertHolidayName(
            name,
            "1985-09-29",
            "1986-09-18",
            "1987-10-07",
            "1988-09-25",
            "1989-09-14",
            "1990-10-03",
            "1991-09-22",
            "1992-09-11",
            "1993-09-30",
            "1994-09-20",
            "1995-09-09",
            "1996-09-27",
            "1997-09-16",
            "1998-10-05",
            "1999-09-24",
        )
        self.assertNoHolidayName(name, range(2000, self.end_year))

    def test_the_day_following_mid_autumn_festival(self):
        name = "中秋節翌日"
        dts = (
            "2020-10-02",
            "2021-09-22",
            "2023-09-30",
            "2024-09-18",
            "2025-10-07",
        )

        # Government Holidays.
        self.assertGovernmentHolidayName(name, dts)
        self.assertGovernmentHolidayName(name, range(2005, self.end_year))
        obs_dts_1 = ("2019-09-16",)
        obs_dts_2 = (
            "2022-09-12",
            "2023-10-03",
        )
        self.assertGovernmentHolidayName(f"{name}後首個工作日", obs_dts_1)
        self.assertGovernmentHolidayName(f"{name}的補假", obs_dts_2)
        self.assertNoGovernmentNonObservedHoliday(obs_dts_1, obs_dts_2)

        # Optional Holidays.
        self.assertOptionalHolidayName(name, dts)
        self.assertOptionalHolidayName(name, self.full_range)

        # Public Holidays.
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, range(2000, self.end_year))
        self.assertNoHolidayName(name, range(1985, 2000))

    def test_new_years_eve(self):
        name = "除夕"
        name_afternoon = f"{name}（下午）"

        # Government Holidays.
        self.assertGovernmentHolidayName(name, "2012-12-31")
        self.assertGovernmentHolidayName(
            name_afternoon,
            (
                f"{year}-12-31"
                for year in (
                    *range(2007, 2011),
                    *range(2013, 2016),
                    *range(2018, 2022),
                    *range(2024, 2026),
                )
            ),
        )
        self.assertNoGovernmentHolidayName(name_afternoon, 2011, 2012, 2016, 2017, 2022, 2023)

        # Optional Holidays.
        self.assertNoOptionalHolidayName(name)
        self.assertNoOptionalHolidayName(name_afternoon)

        self.assertNoHolidayName(name)
        self.assertNoHolidayName(name_afternoon)

    def test_macau_city_day(self):
        name = "澳門市日"
        self.assertNoHolidayName(name)
        self.assertNoGovernmentHolidayName(name)
        self.assertNoOptionalHolidayName(name)

        # M Optional Holidays.
        self.assertSubdivMOptionalHolidayName(
            name, (f"{year}-06-24" for year in range(self.start_year, 2000))
        )
        self.assertNoSubdivMOptionalHolidayName(name, range(2000, self.end_year))
        self.assertNoSubdivIOptionalHolidayName(name)

    def test_day_of_the_municipality_of_ilhas(self):
        name = "海島市日"
        self.assertNoHolidayName(name)
        self.assertNoGovernmentHolidayName(name)
        self.assertNoOptionalHolidayName(name)

        # I Optional Holidays.
        self.assertSubdivIOptionalHolidayName(
            name,
            (f"{year}-11-30" for year in range(self.start_year, 1993)),
            (f"{year}-07-13" for year in range(1993, 2000)),
        )
        self.assertNoSubdivIOptionalHolidayName(name, range(2000, self.end_year))
        self.assertNoSubdivMOptionalHolidayName(name)

    def test_2024_government(self):
        # https://www.gov.mo/en/public-holidays/year-2024/
        dts = (
            # Exemption from work granted to public employees by the Chief Executive.
            "2024-02-09",
            "2024-12-31",
        )
        obs_dts = (
            # Compensatory rest days for public employees set forth in
            # No. 4 of Article 79 of the ETAPM.
            "2024-02-13",
            "2024-02-14",
            "2024-04-01",
            "2024-11-04",
            "2024-12-09",
            "2024-12-23",
        )
        self.assertGovernmentHoliday(dts, obs_dts)
        self.assertNoGovernmentNonObservedHoliday(obs_dts)

    def test_2024_public(self):
        # https://www.dsal.gov.mo/en/text/holiday_table.html
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "元旦"),
            ("2024-02-10", "農曆正月初一"),
            ("2024-02-11", "農曆正月初二"),
            ("2024-02-12", "農曆正月初三"),
            ("2024-04-04", "清明節"),
            ("2024-05-01", "勞動節"),
            ("2024-09-18", "中秋節翌日"),
            ("2024-10-01", "中華人民共和國國慶日"),
            ("2024-10-11", "重陽節"),
            ("2024-12-20", "澳門特別行政區成立紀念日"),
        )

    def test_2017_optional(self):
        # https://www.gov.mo/en/public-holidays/year-2017/
        self.assertOptionalHolidaysInYear(
            2017,
            ("2017-01-01", "元旦"),
            ("2017-01-28", "農曆正月初一"),
            ("2017-01-29", "農曆正月初二"),
            ("2017-01-30", "農曆正月初三"),
            ("2017-04-04", "清明節"),
            ("2017-04-14", "耶穌受難日"),
            ("2017-04-15", "復活節前日"),
            ("2017-05-01", "勞動節"),
            ("2017-05-03", "佛誕節"),
            ("2017-05-30", "端午節"),
            ("2017-10-01", "中華人民共和國國慶日"),
            ("2017-10-02", "中華人民共和國國慶日翌日"),
            ("2017-10-05", "中秋節翌日"),
            ("2017-10-28", "重陽節"),
            ("2017-11-02", "追思節"),
            ("2017-12-08", "聖母無原罪瞻禮"),
            ("2017-12-20", "澳門特別行政區成立紀念日"),
            ("2017-12-22", "冬至"),
            ("2017-12-24", "聖誕節前日"),
            ("2017-12-25", "聖誕節"),
        )

    def test_2018_optional(self):
        # https://www.gov.mo/en/public-holidays/year-2018/
        self.assertOptionalHolidaysInYear(
            2018,
            ("2018-01-01", "元旦"),
            ("2018-02-16", "農曆正月初一"),
            ("2018-02-17", "農曆正月初二"),
            ("2018-02-18", "農曆正月初三"),
            ("2018-03-30", "耶穌受難日"),
            ("2018-03-31", "復活節前日"),
            ("2018-04-05", "清明節"),
            ("2018-05-01", "勞動節"),
            ("2018-05-22", "佛誕節"),
            ("2018-06-18", "端午節"),
            ("2018-09-25", "中秋節翌日"),
            ("2018-10-01", "中華人民共和國國慶日"),
            ("2018-10-02", "中華人民共和國國慶日翌日"),
            ("2018-10-17", "重陽節"),
            ("2018-11-02", "追思節"),
            ("2018-12-08", "聖母無原罪瞻禮"),
            ("2018-12-20", "澳門特別行政區成立紀念日"),
            ("2018-12-22", "冬至"),
            ("2018-12-24", "聖誕節前日"),
            ("2018-12-25", "聖誕節"),
        )

    def test_2019_optional(self):
        # https://www.gov.mo/en/public-holidays/year-2019/
        self.assertOptionalHolidaysInYear(
            2019,
            ("2019-01-01", "元旦"),
            ("2019-02-05", "農曆正月初一"),
            ("2019-02-06", "農曆正月初二"),
            ("2019-02-07", "農曆正月初三"),
            ("2019-04-05", "清明節"),
            ("2019-04-19", "耶穌受難日"),
            ("2019-04-20", "復活節前日"),
            ("2019-05-01", "勞動節"),
            ("2019-05-12", "佛誕節"),
            ("2019-06-07", "端午節"),
            ("2019-09-14", "中秋節翌日"),
            ("2019-10-01", "中華人民共和國國慶日"),
            ("2019-10-02", "中華人民共和國國慶日翌日"),
            ("2019-10-07", "重陽節"),
            ("2019-11-02", "追思節"),
            ("2019-12-08", "聖母無原罪瞻禮"),
            ("2019-12-20", "澳門特別行政區成立紀念日"),
            ("2019-12-22", "冬至"),
            ("2019-12-24", "聖誕節前日"),
            ("2019-12-25", "聖誕節"),
        )

    def test_2020_optional(self):
        # https://www.gov.mo/en/public-holidays/year-2020/
        self.assertOptionalHolidaysInYear(
            2020,
            ("2020-01-01", "元旦"),
            ("2020-01-25", "農曆正月初一"),
            ("2020-01-26", "農曆正月初二"),
            ("2020-01-27", "農曆正月初三"),
            ("2020-04-04", "清明節"),
            ("2020-04-10", "耶穌受難日"),
            ("2020-04-11", "復活節前日"),
            ("2020-04-30", "佛誕節"),
            ("2020-05-01", "勞動節"),
            ("2020-06-25", "端午節"),
            ("2020-10-01", "中華人民共和國國慶日"),
            ("2020-10-02", "中秋節翌日; 中華人民共和國國慶日翌日"),
            ("2020-10-25", "重陽節"),
            ("2020-11-02", "追思節"),
            ("2020-12-08", "聖母無原罪瞻禮"),
            ("2020-12-20", "澳門特別行政區成立紀念日"),
            ("2020-12-21", "冬至"),
            ("2020-12-24", "聖誕節前日"),
            ("2020-12-25", "聖誕節"),
        )

    def test_2021_optional(self):
        # https://www.gov.mo/en/public-holidays/year-2021/
        self.assertOptionalHolidaysInYear(
            2021,
            ("2021-01-01", "元旦"),
            ("2021-02-12", "農曆正月初一"),
            ("2021-02-13", "農曆正月初二"),
            ("2021-02-14", "農曆正月初三"),
            ("2021-04-02", "耶穌受難日"),
            ("2021-04-03", "復活節前日"),
            ("2021-04-04", "清明節"),
            ("2021-05-01", "勞動節"),
            ("2021-05-19", "佛誕節"),
            ("2021-06-14", "端午節"),
            ("2021-09-22", "中秋節翌日"),
            ("2021-10-01", "中華人民共和國國慶日"),
            ("2021-10-02", "中華人民共和國國慶日翌日"),
            ("2021-10-14", "重陽節"),
            ("2021-11-02", "追思節"),
            ("2021-12-08", "聖母無原罪瞻禮"),
            ("2021-12-20", "澳門特別行政區成立紀念日"),
            ("2021-12-21", "冬至"),
            ("2021-12-24", "聖誕節前日"),
            ("2021-12-25", "聖誕節"),
        )

    def test_2022_optional(self):
        # https://www.gov.mo/en/public-holidays/year-2022/
        self.assertOptionalHolidaysInYear(
            2022,
            ("2022-01-01", "元旦"),
            ("2022-02-01", "農曆正月初一"),
            ("2022-02-02", "農曆正月初二"),
            ("2022-02-03", "農曆正月初三"),
            ("2022-04-05", "清明節"),
            ("2022-04-15", "耶穌受難日"),
            ("2022-04-16", "復活節前日"),
            ("2022-05-01", "勞動節"),
            ("2022-05-08", "佛誕節"),
            ("2022-06-03", "端午節"),
            ("2022-09-11", "中秋節翌日"),
            ("2022-10-01", "中華人民共和國國慶日"),
            ("2022-10-02", "中華人民共和國國慶日翌日"),
            ("2022-10-04", "重陽節"),
            ("2022-11-02", "追思節"),
            ("2022-12-08", "聖母無原罪瞻禮"),
            ("2022-12-20", "澳門特別行政區成立紀念日"),
            ("2022-12-22", "冬至"),
            ("2022-12-24", "聖誕節前日"),
            ("2022-12-25", "聖誕節"),
        )

    def test_2023_optional(self):
        # https://www.gov.mo/en/public-holidays/year-2023/
        self.assertOptionalHolidaysInYear(
            2023,
            ("2023-01-01", "元旦"),
            ("2023-01-22", "農曆正月初一"),
            ("2023-01-23", "農曆正月初二"),
            ("2023-01-24", "農曆正月初三"),
            ("2023-04-05", "清明節"),
            ("2023-04-07", "耶穌受難日"),
            ("2023-04-08", "復活節前日"),
            ("2023-05-01", "勞動節"),
            ("2023-05-26", "佛誕節"),
            ("2023-06-22", "端午節"),
            ("2023-09-30", "中秋節翌日"),
            ("2023-10-01", "中華人民共和國國慶日"),
            ("2023-10-02", "中華人民共和國國慶日翌日"),
            ("2023-10-23", "重陽節"),
            ("2023-11-02", "追思節"),
            ("2023-12-08", "聖母無原罪瞻禮"),
            ("2023-12-20", "澳門特別行政區成立紀念日"),
            ("2023-12-22", "冬至"),
            ("2023-12-24", "聖誕節前日"),
            ("2023-12-25", "聖誕節"),
        )

    def test_2024_optional(self):
        # https://www.gov.mo/en/public-holidays/year-2024/
        self.assertOptionalHolidaysInYear(
            2024,
            ("2024-01-01", "元旦"),
            ("2024-02-10", "農曆正月初一"),
            ("2024-02-11", "農曆正月初二"),
            ("2024-02-12", "農曆正月初三"),
            ("2024-03-29", "耶穌受難日"),
            ("2024-03-30", "復活節前日"),
            ("2024-04-04", "清明節"),
            ("2024-05-01", "勞動節"),
            ("2024-05-15", "佛誕節"),
            ("2024-06-10", "端午節"),
            ("2024-09-18", "中秋節翌日"),
            ("2024-10-01", "中華人民共和國國慶日"),
            ("2024-10-02", "中華人民共和國國慶日翌日"),
            ("2024-10-11", "重陽節"),
            ("2024-11-02", "追思節"),
            ("2024-12-08", "聖母無原罪瞻禮"),
            ("2024-12-20", "澳門特別行政區成立紀念日"),
            ("2024-12-21", "冬至"),
            ("2024-12-24", "聖誕節前日"),
            ("2024-12-25", "聖誕節"),
        )

    def test_2025_optional(self):
        # https://www.gov.mo/en/public-holidays/year-2025/
        self.assertOptionalHolidaysInYear(
            2025,
            ("2025-01-01", "元旦"),
            ("2025-01-29", "農曆正月初一"),
            ("2025-01-30", "農曆正月初二"),
            ("2025-01-31", "農曆正月初三"),
            ("2025-04-04", "清明節"),
            ("2025-04-18", "耶穌受難日"),
            ("2025-04-19", "復活節前日"),
            ("2025-05-01", "勞動節"),
            ("2025-05-05", "佛誕節"),
            ("2025-05-31", "端午節"),
            ("2025-10-01", "中華人民共和國國慶日"),
            ("2025-10-02", "中華人民共和國國慶日翌日"),
            ("2025-10-07", "中秋節翌日"),
            ("2025-10-29", "重陽節"),
            ("2025-11-02", "追思節"),
            ("2025-12-08", "聖母無原罪瞻禮"),
            ("2025-12-20", "澳門特別行政區成立紀念日"),
            ("2025-12-21", "冬至"),
            ("2025-12-24", "聖誕節前日"),
            ("2025-12-25", "聖誕節"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "元旦"),
            ("2024-02-09", "農曆除夕（下午）"),
            ("2024-02-10", "農曆正月初一"),
            ("2024-02-11", "農曆正月初二"),
            ("2024-02-12", "農曆正月初三"),
            ("2024-02-13", "農曆正月初一的補假"),
            ("2024-02-14", "農曆正月初二的補假"),
            ("2024-03-29", "耶穌受難日"),
            ("2024-03-30", "復活節前日"),
            ("2024-04-01", "復活節前日的補假"),
            ("2024-04-04", "清明節"),
            ("2024-05-01", "勞動節"),
            ("2024-05-15", "佛誕節"),
            ("2024-06-10", "端午節"),
            ("2024-09-18", "中秋節翌日"),
            ("2024-10-01", "中華人民共和國國慶日"),
            ("2024-10-02", "中華人民共和國國慶日翌日"),
            ("2024-10-11", "重陽節"),
            ("2024-11-02", "追思節"),
            ("2024-11-04", "追思節的補假"),
            ("2024-12-08", "聖母無原罪瞻禮"),
            ("2024-12-09", "聖母無原罪瞻禮的補假"),
            ("2024-12-20", "澳門特別行政區成立紀念日"),
            ("2024-12-21", "冬至"),
            ("2024-12-23", "冬至的補假"),
            ("2024-12-24", "聖誕節前日"),
            ("2024-12-25", "聖誕節"),
            ("2024-12-31", "除夕（下午）"),
        )

    def test_l10n_en_mo(self):
        self.assertLocalizedHolidays(
            "en_MO",
            ("2024-01-01", "New Year's Day"),
            ("2024-02-09", "Lunar New Year's Eve (Afternoon)"),
            ("2024-02-10", "Lunar New Year's Day"),
            ("2024-02-11", "The second day of Lunar New Year"),
            ("2024-02-12", "The third day of Lunar New Year"),
            ("2024-02-13", "Compensatory rest day for Lunar New Year's Day"),
            ("2024-02-14", "Compensatory rest day for The second day of Lunar New Year"),
            ("2024-03-29", "Good Friday"),
            ("2024-03-30", "The Day before Easter"),
            ("2024-04-01", "Compensatory rest day for The Day before Easter"),
            ("2024-04-04", "Ching Ming Festival"),
            ("2024-05-01", "Labour Day"),
            ("2024-05-15", "The Buddha's Birthday (Feast of Buddha)"),
            ("2024-06-10", "Tung Ng Festival (Dragon Boat Festival)"),
            ("2024-09-18", "The Day following Chong Chao (Mid-Autumn) Festival"),
            ("2024-10-01", "National Day of the People's Republic of China"),
            ("2024-10-02", "The day following National Day of the People's Republic of China"),
            ("2024-10-11", "Chung Yeung Festival (Festival of Ancestors)"),
            ("2024-11-02", "All Soul's Day"),
            ("2024-11-04", "Compensatory rest day for All Soul's Day"),
            ("2024-12-08", "Immaculate Conception"),
            ("2024-12-09", "Compensatory rest day for Immaculate Conception"),
            ("2024-12-20", "Macao S.A.R. Establishment Day"),
            ("2024-12-21", "Winter Solstice"),
            ("2024-12-23", "Compensatory rest day for Winter Solstice"),
            ("2024-12-24", "Christmas Eve"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-31", "New Year's Eve (Afternoon)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-02-09", "Chinese New Year's Eve (Afternoon)"),
            ("2024-02-10", "Chinese New Year's Day"),
            ("2024-02-11", "The second day of Chinese New Year"),
            ("2024-02-12", "The third day of Chinese New Year"),
            ("2024-02-13", "Compensatory rest day for Chinese New Year's Day"),
            ("2024-02-14", "Compensatory rest day for The second day of Chinese New Year"),
            ("2024-03-29", "Good Friday"),
            ("2024-03-30", "The Day before Easter"),
            ("2024-04-01", "Compensatory rest day for The Day before Easter"),
            ("2024-04-04", "Tomb-Sweeping Day"),
            ("2024-05-01", "Labor Day"),
            ("2024-05-15", "The Buddha's Birthday"),
            ("2024-06-10", "Dragon Boat Festival"),
            ("2024-09-18", "The Day following Mid-Autumn Festival"),
            ("2024-10-01", "National Day of the People's Republic of China"),
            ("2024-10-02", "The day following National Day of the People's Republic of China"),
            ("2024-10-11", "Double Ninth Festival"),
            ("2024-11-02", "All Soul's Day"),
            ("2024-11-04", "Compensatory rest day for All Soul's Day"),
            ("2024-12-08", "Immaculate Conception"),
            ("2024-12-09", "Compensatory rest day for Immaculate Conception"),
            ("2024-12-20", "Macao S.A.R. Establishment Day"),
            ("2024-12-21", "Winter Solstice"),
            ("2024-12-23", "Compensatory rest day for Winter Solstice"),
            ("2024-12-24", "Christmas Eve"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-31", "New Year's Eve (Afternoon)"),
        )

    def test_l10n_pt_mo(self):
        self.assertLocalizedHolidays(
            "pt_MO",
            ("2024-01-01", "Fraternidade Universal"),
            ("2024-02-09", "Véspera do Novo Ano Lunar (na parte da tarde)"),
            ("2024-02-10", "1.º dia do Novo Ano Lunar"),
            ("2024-02-11", "2.º dia do Novo Ano Lunar"),
            ("2024-02-12", "3.º dia do Novo Ano Lunar"),
            ("2024-02-13", "Dia de descanso compensatório relativo ao 1.º dia do Novo Ano Lunar"),
            ("2024-02-14", "Dia de descanso compensatório relativo ao 2.º dia do Novo Ano Lunar"),
            ("2024-03-29", "Morte de Cristo"),
            ("2024-03-30", "Véspera da Ressurreição de Cristo"),
            (
                "2024-04-01",
                "Dia de descanso compensatório relativo ao Véspera da Ressurreição de Cristo",
            ),
            ("2024-04-04", "Cheng Ming (Dia de Finados)"),
            ("2024-05-01", "Dia do Trabalhador"),
            ("2024-05-15", "Dia do Buda"),
            ("2024-06-10", "Tung Ng (Barco Dragão)"),
            ("2024-09-18", "Dia seguinte ao Chong Chao (Bolo Lunar)"),
            ("2024-10-01", "Implantação da República Popular da China"),
            ("2024-10-02", "Dia seguinte à Implantação da República Popular da China"),
            ("2024-10-11", "Chong Yeong (Culto dos Antepassados)"),
            ("2024-11-02", "Dia de Finados"),
            ("2024-11-04", "Dia de descanso compensatório relativo ao Dia de Finados"),
            ("2024-12-08", "Imaculada Conceição"),
            ("2024-12-09", "Dia de descanso compensatório relativo ao Imaculada Conceição"),
            (
                "2024-12-20",
                "Dia Comemorativo do Estabelecimento da Região Administrativa Especial de Macau",
            ),
            ("2024-12-21", "Solstício de Inverno"),
            ("2024-12-23", "Dia de descanso compensatório relativo ao Solstício de Inverno"),
            ("2024-12-24", "Véspera de Natal"),
            ("2024-12-25", "Natal"),
            ("2024-12-31", "Véspera do Dia da Fraternidade Universal (na parte da tarde)"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2024-01-01", "วันขึ้นปีใหม่"),
            ("2024-02-09", "วันก่อนวันตรุษจีน (ครึ่งบ่าย)"),
            ("2024-02-10", "วันตรุษจีน"),
            ("2024-02-11", "วันตรุษจีนวันที่สอง"),
            ("2024-02-12", "วันตรุษจีนวันที่สาม"),
            ("2024-02-13", "ชดเชยวันตรุษจีน"),
            ("2024-02-14", "ชดเชยวันตรุษจีนวันที่สอง"),
            ("2024-03-29", "วันศุกร์ประเสริฐ"),
            ("2024-03-30", "วันก่อนวันอาทิตย์อีสเตอร์"),
            ("2024-04-01", "ชดเชยวันก่อนวันอาทิตย์อีสเตอร์"),
            ("2024-04-04", "วันเช็งเม้ง"),
            ("2024-05-01", "วันแรงงาน"),
            ("2024-05-15", "วันวิสาขบูชา"),
            ("2024-06-10", "วันไหว้บ๊ะจ่าง"),
            ("2024-09-18", "วันหลังวันไหว้พระจันทร์"),
            ("2024-10-01", "วันชาติจีน"),
            ("2024-10-02", "วันหลังวันชาติจีน"),
            ("2024-10-11", "วันไหว้บรรพบุรุษ"),
            ("2024-11-02", "วันภาวนาอุทิศแด่ผู้ล่วงลับ"),
            ("2024-11-04", "ชดเชยวันภาวนาอุทิศแด่ผู้ล่วงลับ"),
            ("2024-12-08", "วันสมโภชแม่พระผู้ปฏิสนธินิรมล"),
            ("2024-12-09", "ชดเชยวันสมโภชแม่พระผู้ปฏิสนธินิรมล"),
            ("2024-12-20", "วันสถาปนาเขตบริหารพิเศษมาเก๊า"),
            ("2024-12-21", "วันตงจื้อ(เหมายัน)"),
            ("2024-12-23", "ชดเชยวันตงจื้อ(เหมายัน)"),
            ("2024-12-24", "วันคริสต์มาสอีฟ"),
            ("2024-12-25", "วันคริสต์มาส"),
            ("2024-12-31", "วันสิ้นปี (ครึ่งบ่าย)"),
        )

    def test_l10n_zh_cn(self):
        self.assertLocalizedHolidays(
            "zh_CN",
            ("2024-01-01", "元旦"),
            ("2024-02-09", "农历除夕（下午）"),
            ("2024-02-10", "农历正月初一"),
            ("2024-02-11", "农历正月初二"),
            ("2024-02-12", "农历正月初三"),
            ("2024-02-13", "农历正月初一的补假"),
            ("2024-02-14", "农历正月初二的补假"),
            ("2024-03-29", "耶稣受难日"),
            ("2024-03-30", "复活节前日"),
            ("2024-04-01", "复活节前日的补假"),
            ("2024-04-04", "清明节"),
            ("2024-05-01", "劳动节"),
            ("2024-05-15", "佛诞节"),
            ("2024-06-10", "端午节"),
            ("2024-09-18", "中秋节翌日"),
            ("2024-10-01", "中华人民共和国国庆日"),
            ("2024-10-02", "中华人民共和国国庆日翌日"),
            ("2024-10-11", "重阳节"),
            ("2024-11-02", "追思节"),
            ("2024-11-04", "追思节的补假"),
            ("2024-12-08", "圣母无原罪瞻礼"),
            ("2024-12-09", "圣母无原罪瞻礼的补假"),
            ("2024-12-20", "澳门特别行政区成立纪念日"),
            ("2024-12-21", "冬至"),
            ("2024-12-23", "冬至的补假"),
            ("2024-12-24", "圣诞节前日"),
            ("2024-12-25", "圣诞节"),
            ("2024-12-31", "除夕（下午）"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.sierra_leone import SierraLeone
from tests.common import CommonCountryTests


class TestSierraLeone(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(1962, 2050)
        super().setUpClass(SierraLeone, years=years, years_non_observed=years)
        cls.no_estimated_holidays = SierraLeone(years=years, islamic_show_estimated=False)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(1962, 2050)))
        dt = (
            "2011-01-03",
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_armed_forces_day(self):
        name = "Armed Forces Day"
        self.assertHolidayName(name, (f"{year}-02-18" for year in range(2002, 2050)))
        self.assertNoHolidayName(name, range(1962, 2002))
        dt = (
            "2006-02-20",
            "2012-02-20",
            "2017-02-20",
            "2018-02-19",
            "2023-02-20",
            "2024-02-19",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_international_womens_day(self):
        name = "International Women's Day"
        self.assertHolidayName(name, (f"{year}-03-08" for year in range(2018, 2050)))
        self.assertNoHolidayName(name, range(1962, 2018))
        dt = (
            "2020-03-09",
            "2025-03-10",
            "2026-03-09",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
        )
        self.assertHolidayName(name, range(1962, 2050))

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2018-04-02",
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
        )
        self.assertHolidayName(name, range(1962, 2050))

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-04-27" for year in range(1962, 2050)))
        dt = (
            "2008-04-28",
            "2013-04-29",
            "2019-04-29",
            "2024-04-29",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_labor_day(self):
        name = "International Worker's Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1962, 2050)))
        dt = (
            "2005-05-02",
            "2010-05-03",
            "2011-05-02",
            "2016-05-02",
            "2021-05-03",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in range(1962, 2050)))
        dt = (
            "2005-12-27",
            "2010-12-27",
            "2016-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in range(1962, 2050)))
        dt = (
            "2009-12-28",
            "2015-12-28",
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_prophets_birthday(self):
        name = "Prophet's Birthday"
        self.assertHolidayName(
            name,
            "2018-11-21",
            "2019-11-10",
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(1962, 2050))
        dt = (
            "2015-01-05",
            "2016-12-12",
            "2019-11-11",
            "2022-10-10",
            "2024-09-16",
        )
        self.assertHolidayName(f"{name} (observed)", self.no_estimated_holidays, dt)
        self.assertNoNonObservedHoliday(dt)

    def test_eid_al_fitr(self):
        name = "Eid al-Fitr"
        self.assertHolidayName(
            name,
            "2018-06-15",
            "2019-06-05",
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(1962, 2050))
        dt = (
            "2012-08-20",
            "2017-06-26",
            "2020-05-25",
            "2025-03-31",
        )
        self.assertHolidayName(f"{name} (observed)", self.no_estimated_holidays, dt)
        self.assertNoNonObservedHoliday(dt)

    def test_eid_al_adha(self):
        name = "Eid al-Adha"
        self.assertHolidayName(
            name,
            "2018-08-22",
            "2019-08-12",
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(1962, 2050))
        dt = (
            "2011-11-07",
            "2014-10-06",
            "2016-09-12",
            "2022-07-11",
            "2024-06-17",
        )
        self.assertHolidayName(f"{name} (observed)", self.no_estimated_holidays, dt)
        self.assertNoNonObservedHoliday(dt)

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-02-18", "Armed Forces Day"),
            ("2024-02-19", "Armed Forces Day (observed)"),
            ("2024-03-08", "International Women's Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-04-10", "Eid al-Fitr"),
            ("2024-04-27", "Independence Day"),
            ("2024-04-29", "Independence Day (observed)"),
            ("2024-05-01", "International Worker's Day"),
            ("2024-06-16", "Eid al-Adha"),
            ("2024-06-17", "Eid al-Adha (observed)"),
            ("2024-09-15", "Prophet's Birthday"),
            ("2024-09-16", "Prophet's Birthday (observed)"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "New Year's Day"),
            ("2025-02-18", "Armed Forces Day"),
            ("2025-03-08", "International Women's Day"),
            ("2025-03-10", "International Women's Day (observed)"),
            ("2025-03-30", "Eid al-Fitr (estimated)"),
            ("2025-03-31", "Eid al-Fitr (observed, estimated)"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-04-27", "Independence Day"),
            ("2025-04-28", "Independence Day (observed)"),
            ("2025-05-01", "International Worker's Day"),
            ("2025-06-06", "Eid al-Adha (estimated)"),
            ("2025-09-04", "Prophet's Birthday (estimated)"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-02-18", "Armed Forces Day"),
            ("2025-03-08", "International Women's Day"),
            ("2025-03-10", "International Women's Day (observed)"),
            ("2025-03-30", "Eid al-Fitr (estimated)"),
            ("2025-03-31", "Eid al-Fitr (observed, estimated)"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-04-27", "Independence Day"),
            ("2025-04-28", "Independence Day (observed)"),
            ("2025-05-01", "Labor Day"),
            ("2025-06-06", "Eid al-Adha (estimated)"),
            ("2025-09-04", "Prophet's Birthday (estimated)"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.belgium import Belgium
from tests.common import CommonCountryTests


class TestBelgium(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Belgium)

    def test_new_years_day(self):
        self.assertHolidayName("Nieuwjaar", (f"{year}-01-01" for year in self.full_range))

    def test_good_friday(self):
        name = "Goede Vrijdag"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertBankHolidayName(name, self.full_range)

    def test_easter_sunday(self):
        name = "Pasen"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Paasmaandag"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        self.assertHolidayName("Dag van de Arbeid", (f"{year}-05-01" for year in self.full_range))

    def test_ascension_day(self):
        name = "O. L. H. Hemelvaart"
        self.assertHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_friday_after_ascension_day(self):
        name = "Vrijdag na O. L. H. Hemelvaart"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(
            name,
            "2020-05-22",
            "2021-05-14",
            "2022-05-27",
            "2023-05-19",
            "2024-05-10",
            "2025-05-30",
        )
        self.assertBankHolidayName(name, self.full_range)

    def test_whit_sunday(self):
        name = "Pinksteren"
        self.assertHolidayName(
            name,
            "2020-05-31",
            "2021-05-23",
            "2022-06-05",
            "2023-05-28",
            "2024-05-19",
            "2025-06-08",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_monday(self):
        name = "Pinkstermaandag"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_national_day(self):
        self.assertHolidayName("Nationale feestdag", (f"{year}-07-21" for year in self.full_range))

    def test_assumption_day(self):
        self.assertHolidayName(
            "O. L. V. Hemelvaart", (f"{year}-08-15" for year in self.full_range)
        )

    def test_all_saints_day(self):
        self.assertHolidayName("Allerheiligen", (f"{year}-11-01" for year in self.full_range))

    def test_armistice_day(self):
        self.assertHolidayName("Wapenstilstand", (f"{year}-11-11" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName("Kerstmis", (f"{year}-12-25" for year in self.full_range))

    def test_bank_holiday(self):
        name = "Banksluitingsdag"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(name, (f"{year}-12-26" for year in self.full_range))

    def test_2020(self):
        self.assertHolidaysInYear(
            2020,
            ("2020-01-01", "Nieuwjaar"),
            ("2020-04-12", "Pasen"),
            ("2020-04-13", "Paasmaandag"),
            ("2020-05-01", "Dag van de Arbeid"),
            ("2020-05-21", "O. L. H. Hemelvaart"),
            ("2020-05-31", "Pinksteren"),
            ("2020-06-01", "Pinkstermaandag"),
            ("2020-07-21", "Nationale feestdag"),
            ("2020-08-15", "O. L. V. Hemelvaart"),
            ("2020-11-01", "Allerheiligen"),
            ("2020-11-11", "Wapenstilstand"),
            ("2020-12-25", "Kerstmis"),
        )

    def test_2021(self):
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "Nieuwjaar"),
            ("2021-04-04", "Pasen"),
            ("2021-04-05", "Paasmaandag"),
            ("2021-05-01", "Dag van de Arbeid"),
            ("2021-05-13", "O. L. H. Hemelvaart"),
            ("2021-05-23", "Pinksteren"),
            ("2021-05-24", "Pinkstermaandag"),
            ("2021-07-21", "Nationale feestdag"),
            ("2021-08-15", "O. L. V. Hemelvaart"),
            ("2021-11-01", "Allerheiligen"),
            ("2021-11-11", "Wapenstilstand"),
            ("2021-12-25", "Kerstmis"),
        )

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Nieuwjaar"),
            ("2022-04-17", "Pasen"),
            ("2022-04-18", "Paasmaandag"),
            ("2022-05-01", "Dag van de Arbeid"),
            ("2022-05-26", "O. L. H. Hemelvaart"),
            ("2022-06-05", "Pinksteren"),
            ("2022-06-06", "Pinkstermaandag"),
            ("2022-07-21", "Nationale feestdag"),
            ("2022-08-15", "O. L. V. Hemelvaart"),
            ("2022-11-01", "Allerheiligen"),
            ("2022-11-11", "Wapenstilstand"),
            ("2022-12-25", "Kerstmis"),
        )

    def test_2022_bank(self):
        self.assertBankHolidaysInYear(
            2022,
            ("2022-04-15", "Goede Vrijdag"),
            ("2022-05-27", "Vrijdag na O. L. H. Hemelvaart"),
            ("2022-12-26", "Banksluitingsdag"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Nieuwjaar"),
            ("2022-04-15", "Goede Vrijdag"),
            ("2022-04-17", "Pasen"),
            ("2022-04-18", "Paasmaandag"),
            ("2022-05-01", "Dag van de Arbeid"),
            ("2022-05-26", "O. L. H. Hemelvaart"),
            ("2022-05-27", "Vrijdag na O. L. H. Hemelvaart"),
            ("2022-06-05", "Pinksteren"),
            ("2022-06-06", "Pinkstermaandag"),
            ("2022-07-21", "Nationale feestdag"),
            ("2022-08-15", "O. L. V. Hemelvaart"),
            ("2022-11-01", "Allerheiligen"),
            ("2022-11-11", "Wapenstilstand"),
            ("2022-12-25", "Kerstmis"),
            ("2022-12-26", "Banksluitingsdag"),
        )

    def test_l10n_de(self):
        self.assertLocalizedHolidays(
            "de",
            ("2022-01-01", "Neujahr"),
            ("2022-04-15", "Karfreitag"),
            ("2022-04-17", "Ostern"),
            ("2022-04-18", "Ostermontag"),
            ("2022-05-01", "Tag der Arbeit"),
            ("2022-05-26", "Christi Himmelfahrt"),
            ("2022-05-27", "Freitag nach Christi Himmelfahrt"),
            ("2022-06-05", "Pfingsten"),
            ("2022-06-06", "Pfingstmontag"),
            ("2022-07-21", "Nationalfeiertag"),
            ("2022-08-15", "Mariä Himmelfahrt"),
            ("2022-11-01", "Allerheiligen"),
            ("2022-11-11", "Waffenstillstand"),
            ("2022-12-25", "Weihnachten"),
            ("2022-12-26", "Bankschlusstag"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-05-27", "Friday after Ascension Day"),
            ("2022-06-05", "Whit Sunday"),
            ("2022-06-06", "Whit Monday"),
            ("2022-07-21", "National Day"),
            ("2022-08-15", "Assumption Day"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-11-11", "Armistice Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Bank Holiday"),
        )

    def test_l10n_fr(self):
        self.assertLocalizedHolidays(
            "fr",
            ("2022-01-01", "Nouvel An"),
            ("2022-04-15", "Vendredi Saint"),
            ("2022-04-17", "Pâques"),
            ("2022-04-18", "Lundi de Pâques"),
            ("2022-05-01", "Fête du Travail"),
            ("2022-05-26", "Ascension"),
            ("2022-05-27", "Vendredi suivant l'Ascension"),
            ("2022-06-05", "Pentecôte"),
            ("2022-06-06", "Lundi de Pentecôte"),
            ("2022-07-21", "Fête nationale"),
            ("2022-08-15", "Assomption"),
            ("2022-11-01", "Toussaint"),
            ("2022-11-11", "Jour de l'Armistice"),
            ("2022-12-25", "Noël"),
            ("2022-12-26", "Jour de fermeture bancaire"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-17", "Великдень"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-05-01", "День праці"),
            ("2022-05-26", "Вознесіння Господнє"),
            ("2022-05-27", "Пʼятниця після Вознесіння Господнього"),
            ("2022-06-05", "Трійця"),
            ("2022-06-06", "День Святого Духа"),
            ("2022-07-21", "Національне свято"),
            ("2022-08-15", "Внебовзяття Пресвятої Діви Марії"),
            ("2022-11-01", "День усіх святих"),
            ("2022-11-11", "День перемирʼя"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Банківський вихідний"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import BANK, PUBLIC
from holidays.countries.tanzania import Tanzania
from tests.common import CommonCountryTests


class TestTanzania(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Tanzania)

    def test_special_holidays(self):
        self.assertHoliday(
            "1967-12-01",
            "2002-08-25",
            "2015-11-05",
            "2020-10-28",
            "2021-03-22",
            "2021-03-25",
            "2022-08-23",
        )

    def test_new_years_day(self):
        name = "Mwaka Mpya"
        self.assertHolidayName(
            name,
            (
                f"{year}-01-01"
                for year in (*range(self.start_year, 1967), *range(1987, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, range(1967, 1987))
        obs_dts = (
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
        )
        self.assertHolidayName(f"Badala ya {name}", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_zanzibar_revolution_day(self):
        name = "Mapinduzi ya Zanzibar"
        self.assertHolidayName(name, (f"{year}-01-12" for year in self.full_range))
        obs_dts = (
            "2014-01-13",
            "2019-01-14",
            "2020-01-13",
        )
        self.assertHolidayName(f"Badala ya {name}", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_party_founding_day(self):
        name_asp = "Kuzaliwa kwa ASP"
        name_ccm = "Kuzaliwa kwa Chama cha Mapinduzi"
        self.assertHolidayName(name_asp, (f"{year}-02-05" for year in range(1973, 1977)))
        self.assertHolidayName(name_ccm, (f"{year}-02-05" for year in range(1977, 1994)))
        self.assertNoHolidayName(
            name_asp, range(self.start_year, 1973), range(1977, self.end_year)
        )
        self.assertNoHolidayName(
            name_ccm, range(self.start_year, 1977), range(1994, self.end_year)
        )

    def test_sheikh_abeid_amani_karume_day(self):
        name = (
            "Siku ya kumbukumbu ya Rais wa Kwanza wa Serikali ya "
            "Mapinduzi Zanzibar Sheikh Abeid Amani Karume"
        )
        self.assertHolidayName(name, (f"{year}-04-07" for year in range(2006, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2006))
        obs_dts = (
            "2012-04-10",
            "2013-04-08",
            "2018-04-09",
            "2019-04-08",
        )
        self.assertHolidayName(f"Badala ya {name}", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Ijumaa Kuu"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_sunday(self):
        name = "Sikukuu ya Pasaka"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertBankHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Jumatatu ya Pasaka"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_union_celebrations(self):
        name_day = "Sikukuu ya Muungano"
        name_celebrations = "Muungano wa Tanzania"
        self.assertHolidayName(
            name_day, (f"{year}-04-26" for year in range(self.start_year, 1991))
        )
        self.assertHolidayName(
            name_celebrations, (f"{year}-04-26" for year in range(1991, self.end_year))
        )
        self.assertNoHolidayName(name_day, range(1991, self.end_year))
        self.assertNoHolidayName(name_celebrations, range(self.start_year, 1991))
        obs_dts = (
            "2008-04-28",
            "2009-04-27",
            "2014-04-28",
            "2015-04-27",
            "2020-04-27",
        )
        self.assertHolidayName(f"Badala ya {name_celebrations}", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_international_workers_day(self):
        name = "Sikukuu ya Wafanyakazi Ulimwenguni"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2010-05-03",
            "2011-05-02",
            "2016-05-02",
            "2021-05-03",
            "2022-05-02",
        )
        self.assertHolidayName(f"Badala ya {name}", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_international_trade_fair_and_peasants_day(self):
        name_1965 = "Saba Saba"
        name_1977 = "Sikukuu ya Wakulima"
        name_1994 = "Sabasaba"

        self.assertHolidayName(
            name_1965, (f"{year}-07-07" for year in range(self.start_year, 1977))
        )
        self.assertHolidayName(
            name_1977,
            (f"{year}-07-07" for year in range(1977, 1994)),
            (f"{year}-08-08" for year in range(1995, self.end_year)),
        )
        self.assertHolidayName(name_1994, (f"{year}-07-07" for year in range(1994, self.end_year)))
        self.assertNoHolidayName(name_1965, range(1977, self.end_year))
        self.assertNoHolidayName(name_1977, range(self.start_year, 1977), 1994)
        self.assertNoHolidayName(name_1994, range(self.start_year, 1994))

        obs_dts_jul7 = (
            "2007-07-09",
            "2012-07-09",
            "2013-07-08",
            "2018-07-09",
            "2019-07-08",
        )
        self.assertHolidayName(f"Badala ya {name_1994}", obs_dts_jul7)

        obs_dts_aug8 = (
            "2009-08-10",
            "2010-08-09",
            "2015-08-10",
            "2020-08-10",
            "2021-08-09",
        )
        self.assertHolidayName(f"Badala ya {name_1977}", obs_dts_aug8)
        self.assertNoNonObservedHoliday(obs_dts_jul7, obs_dts_aug8)

    def test_the_mwalimu_nyerere_day_and_the_climax_of_the_uhuru_torch_race(self):
        name = "Kumbukumbu ya Mwalimu Nyerere na Kilele cha mbio za Mwenge"
        self.assertHolidayName(name, (f"{year}-10-14" for year in range(2002, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2002))
        obs_dts = (
            "2006-10-16",
            "2007-10-15",
            "2012-10-15",
            "2017-10-16",
            "2018-10-15",
        )
        self.assertHolidayName(f"Badala ya {name}", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_and_republic_day(self):
        name = "Uhuru na Jamhuri"
        self.assertHolidayName(name, (f"{year}-12-09" for year in self.full_range))
        obs_dts = (
            "2006-12-11",
            "2007-12-10",
            "2012-12-10",
            "2017-12-11",
            "2018-12-10",
        )
        self.assertHolidayName(f"Badala ya {name}", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Kuzaliwa Kristo"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "1994-12-27",
            "2005-12-27",
            "2010-12-27",
            "2011-12-27",
            "2016-12-27",
            "2021-12-27",
        )
        self.assertHolidayName(f"Badala ya {name}", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Siku ya Kupeana Zawadi"
        self.assertHolidayName(
            name,
            (
                f"{year}-12-26"
                for year in (*range(self.start_year, 1966), *range(1993, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, range(1966, 1993))
        obs_dts = (
            "2009-12-28",
            "2010-12-27",
            "2015-12-28",
            "2020-12-28",
            "2021-12-27",
        )
        self.assertHolidayName(f"Badala ya {name}", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_fitr(self):
        name = "Eid El-Fitri"
        self.assertHolidayName(
            name,
            "1965-02-03",
            "1965-02-04",
            "2020-05-24",
            "2020-05-25",
            "2021-05-14",
            "2021-05-15",
            "2022-05-03",
            "2022-05-04",
            "2023-04-22",
            "2023-04-23",
            "2024-04-10",
            "2024-04-11",
            "2025-03-31",
            "2025-04-01",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2018-06-18",
            "2020-05-26",
            "2021-05-17",
        )
        self.assertHolidayName(f"Badala ya {name}", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_adha(self):
        name = "Eid El Hajj"
        self.assertHolidayName(
            name,
            "1965-04-13",
            "1965-04-14",
            "2020-07-31",
            "2021-07-21",
            "2022-07-10",
            "2023-06-29",
            "2024-06-17",
            "2025-06-07",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2014-10-06",
            "2017-09-04",
            "2022-07-11",
        )
        self.assertHolidayName(f"Badala ya {name}", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_prophets_birthday(self):
        name = "Maulidi"
        self.assertHolidayName(
            name,
            "1965-07-12",
            "2020-10-29",
            "2021-10-19",
            "2022-10-09",
            "2023-09-28",
            "2024-09-16",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2015-01-05",
            "2019-11-11",
        )
        self.assertHolidayName(f"Badala ya {name}", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2022_all(self):
        self.assertHolidays(
            Tanzania(categories=(BANK, PUBLIC), years=2022),
            ("2022-01-01", "Mwaka Mpya"),
            ("2022-01-03", "Badala ya Mwaka Mpya"),
            ("2022-01-12", "Mapinduzi ya Zanzibar"),
            (
                "2022-04-07",
                "Siku ya kumbukumbu ya Rais wa Kwanza wa Serikali ya "
                "Mapinduzi Zanzibar Sheikh Abeid Amani Karume",
            ),
            ("2022-04-15", "Ijumaa Kuu"),
            ("2022-04-17", "Sikukuu ya Pasaka"),
            ("2022-04-18", "Jumatatu ya Pasaka"),
            ("2022-04-26", "Muungano wa Tanzania"),
            ("2022-05-01", "Sikukuu ya Wafanyakazi Ulimwenguni"),
            ("2022-05-02", "Badala ya Sikukuu ya Wafanyakazi Ulimwenguni"),
            ("2022-05-03", "Eid El-Fitri"),
            ("2022-05-04", "Eid El-Fitri"),
            ("2022-07-07", "Sabasaba"),
            ("2022-07-10", "Eid El Hajj"),
            ("2022-07-11", "Badala ya Eid El Hajj"),
            ("2022-08-08", "Sikukuu ya Wakulima"),
            ("2022-08-23", "Siku ya Sensa ya Kitaifa ya Watu na Makazi"),
            ("2022-10-09", "Maulidi"),
            ("2022-10-14", "Kumbukumbu ya Mwalimu Nyerere na Kilele cha mbio za Mwenge"),
            ("2022-12-09", "Uhuru na Jamhuri"),
            ("2022-12-25", "Kuzaliwa Kristo"),
            ("2022-12-26", "Siku ya Kupeana Zawadi"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "Mwaka Mpya"),
            ("2023-01-12", "Mapinduzi ya Zanzibar"),
            (
                "2023-04-07",
                "Ijumaa Kuu; Siku ya kumbukumbu ya Rais wa Kwanza wa Serikali ya "
                "Mapinduzi Zanzibar Sheikh Abeid Amani Karume",
            ),
            ("2023-04-09", "Sikukuu ya Pasaka"),
            ("2023-04-10", "Jumatatu ya Pasaka"),
            ("2023-04-22", "Eid El-Fitri"),
            ("2023-04-23", "Eid El-Fitri"),
            ("2023-04-26", "Muungano wa Tanzania"),
            ("2023-05-01", "Sikukuu ya Wafanyakazi Ulimwenguni"),
            ("2023-06-29", "Eid El Hajj"),
            ("2023-07-07", "Sabasaba"),
            ("2023-08-08", "Sikukuu ya Wakulima"),
            ("2023-09-28", "Maulidi"),
            ("2023-10-14", "Kumbukumbu ya Mwalimu Nyerere na Kilele cha mbio za Mwenge"),
            ("2023-12-09", "Uhuru na Jamhuri"),
            ("2023-12-25", "Kuzaliwa Kristo"),
            ("2023-12-26", "Siku ya Kupeana Zawadi"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-01-12", "Zanzibar Revolution Day"),
            ("2023-04-07", "Good Friday; The Sheikh Abeid Amani Karume Day"),
            ("2023-04-09", "Easter Sunday"),
            ("2023-04-10", "Easter Monday"),
            ("2023-04-22", "Eid al-Fitr"),
            ("2023-04-23", "Eid al-Fitr"),
            ("2023-04-26", "Union Celebrations"),
            ("2023-05-01", "International Workers' Day"),
            ("2023-06-29", "Eid al-Adha"),
            ("2023-07-07", "International Trade Fair"),
            ("2023-08-08", "Peasants' Day"),
            ("2023-09-28", "Prophet's Birthday"),
            ("2023-10-14", "The Mwalimu Nyerere Day and Climax of the Uhuru Torch Race"),
            ("2023-12-09", "Independence and Republic Day"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

import numpy as np

from holidays.countries.cambodia import Cambodia
from holidays.countries.thailand import Thailand
from holidays.countries.ukraine import Ukraine


class TestNumpy(TestCase):
    def test_years_int_conversion(self):
        years = (1995, 2000)
        years_range = set(range(*years))

        for cls in (Cambodia, Thailand, Ukraine):
            # Test single value.
            for int_x in (np.int16, np.int32, np.int64):
                self.assertEqual(cls(years=int_x(2024)).years, {2024})

            # Test iterable.
            self.assertEqual(cls(years=np.arange(*years)).years, years_range)



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.kyrgyzstan import Kyrgyzstan
from tests.common import CommonCountryTests, WorkingDayTests


class TestKyrgyzstan(CommonCountryTests, WorkingDayTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Kyrgyzstan)

    def test_special_holidays(self):
        self.assertHoliday(
            "2025-01-03",
            "2025-01-06",
        )

    def test_substituted_holidays(self):
        self.assertHoliday(
            "2006-01-09",
            "2007-01-03",
            "2007-01-04",
            "2008-01-02",
            "2008-01-03",
            "2008-01-04",
            "2009-01-02",
            "2009-01-05",
            "2009-01-06",
            "2009-03-23",
            "2009-05-04",
            "2009-12-31",
            "2010-01-04",
            "2010-01-05",
            "2010-01-06",
            "2010-01-08",
            "2010-02-22",
            "2010-08-30",
            "2010-11-15",
            "2011-03-07",
            "2011-05-06",
            "2012-01-03",
            "2012-01-04",
            "2012-01-05",
            "2012-01-06",
            "2013-01-02",
            "2013-01-03",
            "2013-01-04",
            "2013-05-10",
            "2013-08-09",
            "2013-10-14",
            "2013-11-08",
            "2014-01-02",
            "2014-01-03",
            "2014-01-06",
            "2014-07-28",
            "2014-09-05",
            "2015-01-02",
            "2015-01-05",
            "2015-01-06",
            "2016-01-08",
            "2016-05-03",
            "2016-05-04",
            "2017-03-20",
            "2017-05-08",
            "2017-11-06",
            "2018-01-02",
            "2018-01-03",
            "2018-03-09",
            "2018-04-30",
            "2018-08-20",
            "2018-09-03",
            "2018-11-09",
            "2019-01-02",
            "2019-01-03",
            "2019-01-04",
            "2019-05-10",
            "2020-01-02",
            "2020-01-03",
            "2020-01-06",
            "2020-05-04",
            "2021-01-08",
            "2021-07-19",
            "2021-08-30",
            "2022-01-04",
            "2022-01-05",
            "2022-01-06",
            "2022-05-04",
            "2022-05-06",
            "2023-01-03",
            "2023-01-04",
            "2023-01-05",
            "2023-01-06",
            "2023-03-20",
            "2023-05-08",
            "2023-11-06",
            "2024-01-02",
            "2024-01-03",
            "2024-01-04",
            "2024-01-05",
            "2024-03-22",
            "2024-05-02",
            "2024-05-03",
            "2025-01-02",
        )

    def test_workdays(self):
        self.assertWorkingDay(
            "2006-01-15",
            "2006-12-30",
            "2007-01-13",
            "2007-12-29",
            "2007-12-30",
            "2008-01-12",
            "2008-12-27",
            "2009-01-10",
            "2009-01-17",
            "2009-03-28",
            "2009-05-16",
            "2009-12-26",
            "2010-01-16",
            "2010-01-23",
            "2010-01-30",
            "2010-02-06",
            "2010-02-20",
            "2010-08-28",
            "2010-11-13",
            "2011-03-05",
            "2011-05-14",
            "2012-01-14",
            "2012-01-21",
            "2012-01-28",
            "2012-02-04",
            "2012-12-29",
            "2013-01-12",
            "2013-01-19",
            "2013-05-06",
            "2013-08-17",
            "2013-10-12",
            "2013-11-16",
            "2014-01-11",
            "2014-01-18",
            "2014-01-25",
            "2014-08-02",
            "2014-09-01",
            "2015-01-10",
            "2015-02-21",
            "2015-03-23",
            "2016-01-16",
            "2016-04-30",
            "2016-05-07",
            "2017-03-25",
            "2017-05-13",
            "2017-10-28",
            "2017-12-30",
            "2018-01-08",
            "2018-03-03",
            "2018-04-28",
            "2018-09-22",
            "2018-05-07",
            "2018-11-03",
            "2019-02-25",
            "2019-04-08",
            "2019-09-02",
            "2019-05-06",
            "2020-02-24",
            "2020-03-09",
            "2020-03-23",
            "2020-05-11",
            "2021-05-03",
            "2021-07-24",
            "2021-05-10",
            "2022-01-15",
            "2022-02-26",
            "2022-03-26",
            "2022-04-30",
            "2022-05-14",
            "2023-01-09",
            "2023-02-25",
            "2023-03-11",
            "2023-03-25",
            "2023-04-29",
            "2023-05-13",
            "2023-11-11",
            "2024-01-08",
            "2024-01-27",
            "2024-03-02",
            "2024-09-02",
            "2024-04-08",
            "2024-05-06",
            "2024-05-11",
            "2025-01-11",
        )

        for year, dts in {
            2006: (
                "2006-01-15",
                "2006-12-30",
            ),
            2007: (
                "2007-01-13",
                "2007-12-29",
                "2007-12-30",
            ),
            2008: (
                "2008-01-12",
                "2008-12-27",
            ),
            2012: (
                "2012-01-14",
                "2012-01-21",
                "2012-01-28",
                "2012-02-04",
                "2012-12-29",
            ),
            2017: (
                "2017-03-25",
                "2017-05-13",
                "2017-10-28",
                "2017-12-30",
            ),
        }.items():
            self.assertWorkingDay(Kyrgyzstan(years=year), dts)

    def test_new_years_day(self):
        name = "Жаңы жыл"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))

        obs_dts = (
            "2011-01-03",
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (көрүлгөн күнү)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_new_year_holidays(self):
        name = "Жаңы жылдык каникулдар"
        self.assertHolidayName(
            name,
            (f"{year}-01-02" for year in range(2026, self.end_year)),
            (f"{year}-01-03" for year in range(2026, self.end_year)),
            (f"{year}-01-04" for year in range(2026, self.end_year)),
            (f"{year}-01-05" for year in range(2026, self.end_year)),
            (f"{year}-01-06" for year in range(2026, self.end_year)),
        )
        self.assertNoHolidayName(name, range(self.start_year, 2026))

    def test_christmas_day(self):
        name = "Ыйса Пайгамбардын туулган күнү"
        self.assertHolidayName(name, (f"{year}-01-07" for year in self.full_range))

        obs_dts = (
            "1995-01-09",
            "1996-01-08",
            "2001-01-08",
            "2007-01-05",  # special.
            "2017-01-09",
        )
        self.assertHolidayName(f"{name} (көрүлгөн күнү)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_day_of_fatherland_defenders(self):
        name_2005 = "Ата-Журтту коргоонун күнү"
        name_2025 = "Мекенди коргоочулардын күнү"
        self.assertHolidayName(name_2005, (f"{year}-02-23" for year in range(2005, 2025)))
        self.assertNoHolidayName(
            name_2005, range(self.start_year, 2005), range(2025, self.end_year)
        )
        self.assertNoHolidayName(name_2025)

        obs_dts = (
            "2013-02-25",
            "2014-02-24",
        )
        self.assertHolidayName(f"{name_2005} (көрүлгөн күнү)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

        self.assertWorkdayHolidayName(
            name_2025, (f"{year}-02-23" for year in range(2025, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name_2025, range(self.start_year, 2025))
        self.assertNoWorkdayHolidayName(name_2005)

    def test_international_womens_day(self):
        name_1992 = "Аялдар күнү"
        name_2005 = "Аялдардын эл аралык күнү"
        self.assertHolidayName(
            name_1992, (f"{year}-03-08" for year in range(self.start_year, 2005))
        )
        self.assertHolidayName(name_2005, (f"{year}-03-08" for year in range(2005, self.end_year)))
        self.assertNoHolidayName(name_1992, range(2005, self.end_year))
        self.assertNoHolidayName(name_2005, range(self.start_year, 2005))

        obs_dts_1992 = (
            "1992-03-09",
            "1997-03-10",
            "1998-03-09",
            "2003-03-10",
        )
        obs_dts_2005 = (
            "2009-03-09",
            "2014-03-10",
            "2015-03-09",
        )
        self.assertHolidayName(f"{name_1992} (көрүлгөн күнү)", obs_dts_1992)
        self.assertHolidayName(f"{name_2005} (көрүлгөн күнү)", obs_dts_2005)
        self.assertNoNonObservedHoliday(obs_dts_1992, obs_dts_2005)

    def test_nooruz_holiday(self):
        name = "Элдик Нооруз майрамы"
        self.assertHolidayName(name, (f"{year}-03-21" for year in self.full_range))

        obs_dts = (
            "1998-03-23",
            "1999-03-22",
            "2004-03-22",
            "2010-03-22",
            "2021-03-22",
        )
        self.assertHolidayName(f"{name} (көрүлгөн күнү)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_day_of_the_peoples_revolution(self):
        name = "Элдик революция күнү"
        self.assertHolidayName(name, (f"{year}-03-24" for year in range(2008, 2013)))
        self.assertNoHolidayName(name, range(self.start_year, 2008), range(2013, self.end_year))

        obs_dts = ("2012-03-26",)
        self.assertHolidayName(f"{name} (көрүлгөн күнү)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_day_of_the_peoples_april_revolution(self):
        name = "Элдик Апрель революциясы күнү"
        self.assertHolidayName(name, (f"{year}-04-07" for year in range(2016, 2025)))
        self.assertNoHolidayName(name, range(self.start_year, 2016), range(2025, self.end_year))

        obs_dts = ("2018-04-09",)
        self.assertHolidayName(f"{name} (көрүлгөн күнү)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

        self.assertWorkdayHolidayName(
            name, (f"{year}-04-07" for year in range(2025, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2025))

    def test_labor_day(self):
        name_1992 = "Эмгекчилердин эл аралык тилектештик күнү"
        name_1998 = "Эмгек майрамы"
        self.assertHolidayName(
            name_1992, (f"{year}-05-01" for year in range(self.start_year, 1998))
        )
        self.assertHolidayName(name_1998, (f"{year}-05-01" for year in range(1998, self.end_year)))
        self.assertNoHolidayName(name_1992, range(1998, self.end_year))
        self.assertNoHolidayName(name_1998, range(self.start_year, 1998))

        obs_dts_1992 = (
            "1993-05-03",
            "1994-05-02",
        )
        obs_dts_1998 = (
            "2005-05-02",
            "2010-05-03",
            "2011-05-02",
            "2016-05-02",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name_1992} (көрүлгөн күнү)", obs_dts_1992)
        self.assertHolidayName(f"{name_1998} (көрүлгөн күнү)", obs_dts_1998)
        self.assertNoNonObservedHoliday(obs_dts_1992, obs_dts_1998)

    def test_constitution_day(self):
        name = "Кыргыз Республикасынын Конституция күнү"
        self.assertHolidayName(name, (f"{year}-05-05" for year in self.full_range))

        obs_dts = (
            "1996-05-06",
            "2001-05-07",
            "2002-05-06",
            "2012-05-07",
        )
        self.assertHolidayName(f"{name} (көрүлгөн күнү)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_may_holidays(self):
        name = "Май каникулдары"
        self.assertHolidayName(
            name,
            (f"{year}-05-02" for year in range(2025, self.end_year)),
            (f"{year}-05-03" for year in range(2025, self.end_year)),
            (f"{year}-05-04" for year in range(2025, self.end_year)),
            (f"{year}-05-06" for year in range(2025, self.end_year)),
            (f"{year}-05-07" for year in range(2025, self.end_year)),
            (f"{year}-05-08" for year in range(2025, self.end_year)),
        )
        self.assertNoHolidayName(name, range(self.start_year, 2025))

    def test_victory_day(self):
        name = "Жеңиш күнү"
        self.assertHolidayName(name, (f"{year}-05-09" for year in self.full_range))

        obs_dts = (
            "2004-05-10",
            "2009-05-11",
            "2010-05-10",
            "2015-05-11",
        )
        self.assertHolidayName(f"{name} (көрүлгөн күнү)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "Кыргыз Республикасынын Көз карандысыздыгынын күнү"
        self.assertHolidayName(name, (f"{year}-08-31" for year in self.full_range))

        obs_dts = (
            "2002-09-02",
            "2003-09-01",
            "2008-09-01",
            "2013-09-02",
        )
        self.assertHolidayName(f"{name} (көрүлгөн күнү)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_days_of_history_and_commemoration_of_ancestors(self):
        name_2002 = "Улуу Октябрь социалисттик революциясынын күнү"
        name_2018 = "Тарых жана ата-бабаларды эскерүү күндөрү"

        self.assertHolidayName(name_2002, (f"{year}-11-07" for year in range(2002, 2018)))
        self.assertNoHolidayName(
            name_2002, range(self.start_year, 2002), range(2018, self.end_year)
        )
        self.assertHolidayName(
            name_2018,
            (f"{year}-11-07" for year in range(2018, 2025)),
            (f"{year}-11-08" for year in range(2018, 2025)),
        )
        self.assertNoHolidayName(
            name_2018, range(self.start_year, 2018), range(2025, self.end_year)
        )

        obs_dts_2002 = (
            "2004-11-08",
            "2009-11-09",
            "2010-11-08",
            "2015-11-09",
        )
        obs_dts_2018 = (
            "2020-11-09",
            "2020-11-10",
            "2021-11-09",
        )
        self.assertHolidayName(f"{name_2002} (көрүлгөн күнү)", obs_dts_2002)
        self.assertHolidayName(f"{name_2018} (көрүлгөн күнү)", obs_dts_2018)
        self.assertNoNonObservedHoliday(obs_dts_2002, obs_dts_2018)

        self.assertWorkdayHolidayName(
            name_2018,
            (f"{year}-11-07" for year in range(2025, self.end_year)),
            (f"{year}-11-08" for year in range(2025, self.end_year)),
        )
        self.assertNoWorkdayHolidayName(name_2002)
        self.assertNoWorkdayHolidayName(name_2018, range(self.start_year, 2025))

    def test_eid_al_fitr(self):
        name = "Орозо айт"
        self.assertHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-03",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

        obs_dts = (
            "2012-08-20",  # special.
            "2020-05-25",
        )
        self.assertHolidayName(f"{name} (көрүлгөн күнү)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_kurman_ait(self):
        name = "Курман айт"
        self.assertHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-17",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

        obs_dts = (
            "2007-01-02",  # special.
            "2011-11-08",  # special.
            "2014-10-06",
            "2019-08-12",
            "2022-07-11",
        )
        self.assertHolidayName(f"{name} (көрүлгөн күнү)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Жаңы жыл"),
            ("2022-01-03", "Жаңы жыл (көрүлгөн күнү)"),
            ("2022-01-04", "Эс алуу күнү (15.01.2022 күнүнөн которулган)"),
            ("2022-01-05", "Эс алуу күнү (26.02.2022 күнүнөн которулган)"),
            ("2022-01-06", "Эс алуу күнү (26.03.2022 күнүнөн которулган)"),
            ("2022-01-07", "Ыйса Пайгамбардын туулган күнү"),
            ("2022-02-23", "Ата-Журтту коргоонун күнү"),
            ("2022-03-08", "Аялдардын эл аралык күнү"),
            ("2022-03-21", "Элдик Нооруз майрамы"),
            ("2022-04-07", "Элдик Апрель революциясы күнү"),
            ("2022-05-01", "Эмгек майрамы"),
            ("2022-05-02", "Эмгек майрамы (көрүлгөн күнү)"),
            ("2022-05-03", "Орозо айт"),
            ("2022-05-04", "Эс алуу күнү (30.04.2022 күнүнөн которулган)"),
            ("2022-05-05", "Кыргыз Республикасынын Конституция күнү"),
            ("2022-05-06", "Эс алуу күнү (14.05.2022 күнүнөн которулган)"),
            ("2022-05-09", "Жеңиш күнү"),
            ("2022-07-09", "Курман айт"),
            ("2022-07-11", "Курман айт (көрүлгөн күнү)"),
            ("2022-08-31", "Кыргыз Республикасынын Көз карандысыздыгынын күнү"),
            ("2022-11-07", "Тарых жана ата-бабаларды эскерүү күндөрү"),
            ("2022-11-08", "Тарых жана ата-бабаларды эскерүү күндөрү"),
        )

    def test_2025(self):
        self.assertHolidaysInYear(
            2025,
            ("2025-01-01", "Жаңы жыл"),
            ("2025-01-02", "Эс алуу күнү (11.01.2025 күнүнөн которулган)"),
            ("2025-01-03", "Кошумча эс алуу күнү"),
            ("2025-01-06", "Кошумча эс алуу күнү"),
            ("2025-01-07", "Ыйса Пайгамбардын туулган күнү"),
            ("2025-03-08", "Аялдардын эл аралык күнү"),
            ("2025-03-21", "Элдик Нооруз майрамы"),
            ("2025-03-30", "Орозо айт"),
            ("2025-05-01", "Эмгек майрамы"),
            ("2025-05-02", "Май каникулдары"),
            ("2025-05-03", "Май каникулдары"),
            ("2025-05-04", "Май каникулдары"),
            ("2025-05-05", "Кыргыз Республикасынын Конституция күнү"),
            ("2025-05-06", "Май каникулдары"),
            ("2025-05-07", "Май каникулдары"),
            ("2025-05-08", "Май каникулдары"),
            ("2025-05-09", "Жеңиш күнү"),
            ("2025-06-06", "Курман айт"),
            ("2025-08-31", "Кыргыз Республикасынын Көз карандысыздыгынын күнү"),
        )

        self.assertWorkdayHolidaysInYear(
            2025,
            ("2025-02-23", "Мекенди коргоочулардын күнү"),
            ("2025-04-07", "Элдик Апрель революциясы күнү"),
            ("2025-11-07", "Тарых жана ата-бабаларды эскерүү күндөрү"),
            ("2025-11-08", "Тарых жана ата-бабаларды эскерүү күндөрү"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Жаңы жыл"),
            ("2024-01-02", "Эс алуу күнү (08.01.2024 күнүнөн которулган)"),
            ("2024-01-03", "Эс алуу күнү (27.01.2024 күнүнөн которулган)"),
            ("2024-01-04", "Эс алуу күнү (02.03.2024 күнүнөн которулган)"),
            ("2024-01-05", "Эс алуу күнү (02.09.2024 күнүнөн которулган)"),
            ("2024-01-07", "Ыйса Пайгамбардын туулган күнү"),
            ("2024-02-23", "Ата-Журтту коргоонун күнү"),
            ("2024-03-08", "Аялдардын эл аралык күнү"),
            ("2024-03-21", "Элдик Нооруз майрамы"),
            ("2024-03-22", "Эс алуу күнү (08.04.2024 күнүнөн которулган)"),
            ("2024-04-07", "Элдик Апрель революциясы күнү"),
            ("2024-04-10", "Орозо айт"),
            ("2024-05-01", "Эмгек майрамы"),
            ("2024-05-02", "Эс алуу күнү (06.05.2024 күнүнөн которулган)"),
            ("2024-05-03", "Эс алуу күнү (11.05.2024 күнүнөн которулган)"),
            ("2024-05-05", "Кыргыз Республикасынын Конституция күнү"),
            ("2024-05-09", "Жеңиш күнү"),
            ("2024-06-17", "Курман айт"),
            ("2024-08-31", "Кыргыз Республикасынын Көз карандысыздыгынын күнү"),
            ("2024-11-07", "Тарых жана ата-бабаларды эскерүү күндөрү"),
            ("2024-11-08", "Тарых жана ата-бабаларды эскерүү күндөрү"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-01-02", "Day off (substituted from 01/08/2024)"),
            ("2024-01-03", "Day off (substituted from 01/27/2024)"),
            ("2024-01-04", "Day off (substituted from 03/02/2024)"),
            ("2024-01-05", "Day off (substituted from 09/02/2024)"),
            ("2024-01-07", "Christmas Day"),
            ("2024-02-23", "Fatherland Defender's Day"),
            ("2024-03-08", "International Women's Day"),
            ("2024-03-21", "Nooruz Holiday"),
            ("2024-03-22", "Day off (substituted from 04/08/2024)"),
            ("2024-04-07", "Day of the People's April Revolution"),
            ("2024-04-10", "Eid al-Fitr"),
            ("2024-05-01", "Labor Day"),
            ("2024-05-02", "Day off (substituted from 05/06/2024)"),
            ("2024-05-03", "Day off (substituted from 05/11/2024)"),
            ("2024-05-05", "Constitution Day"),
            ("2024-05-09", "Victory Day"),
            ("2024-06-17", "Eid al-Adha"),
            ("2024-08-31", "Independence Day"),
            ("2024-11-07", "Days of History and Commemoration of Ancestors"),
            ("2024-11-08", "Days of History and Commemoration of Ancestors"),
        )

    def test_l10n_ru_kg(self):
        self.assertLocalizedHolidays(
            "ru_KG",
            ("2024-01-01", "Новый год"),
            ("2024-01-02", "Выходной (перенесено с 08.01.2024)"),
            ("2024-01-03", "Выходной (перенесено с 27.01.2024)"),
            ("2024-01-04", "Выходной (перенесено с 02.03.2024)"),
            ("2024-01-05", "Выходной (перенесено с 02.09.2024)"),
            ("2024-01-07", "Рождество Христово"),
            ("2024-02-23", "День защитника Отечества"),
            ("2024-03-08", "Международный женский день"),
            ("2024-03-21", "Народный праздник Нооруз"),
            ("2024-03-22", "Выходной (перенесено с 08.04.2024)"),
            ("2024-04-07", "День народной Апрельской революции"),
            ("2024-04-10", "Орозо айт"),
            ("2024-05-01", "Праздник труда"),
            ("2024-05-02", "Выходной (перенесено с 06.05.2024)"),
            ("2024-05-03", "Выходной (перенесено с 11.05.2024)"),
            ("2024-05-05", "День Конституции Кыргызской Республики"),
            ("2024-05-09", "День Победы"),
            ("2024-06-17", "Курман айт"),
            ("2024-08-31", "День независимости Кыргызской Республики"),
            ("2024-11-07", "Дни истории и памяти предков"),
            ("2024-11-08", "Дни истории и памяти предков"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.financial.ice_futures_europe import IceFuturesEurope
from tests.common import CommonFinancialTests


class TestIceFuturesEurope(CommonFinancialTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(IceFuturesEurope)

    def test_2021(self):
        self.assertHolidays(
            IceFuturesEurope(years=2021),
            ("2021-01-01", "New Year's Day"),
            ("2021-04-02", "Good Friday"),
        )

    def test_2022(self):
        self.assertHolidays(
            IceFuturesEurope(years=2022),
            ("2022-04-15", "Good Friday"),
            ("2022-12-26", "Christmas Day"),
        )

    def test_2023(self):
        self.assertHolidays(
            IceFuturesEurope(years=2023),
            ("2023-01-02", "New Year's Day"),
            ("2023-04-07", "Good Friday"),
            ("2023-12-25", "Christmas Day"),
        )

    def test_2024(self):
        self.assertHolidays(
            IceFuturesEurope(years=2024),
            ("2024-01-01", "New Year's Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

"""Generate index.md and contributing.md with proper link."""

import re
from pathlib import Path

import mkdocs_gen_files


GFM_MKDOCS_MAPPING = {
    "note": "note",
    "tip": "tip",
    "important": "important",
    "warning": "warning",
    "caution": "danger",
}

FILES_REPLACE_MAPPING = {
    "examples.md": (
        (
            "[`HolidayBase`](https://github.com/vacanza/holidays/blob/main/holidays/holiday_base.py)",
            "[`HolidayBase`][holidays.HolidayBase]",
        ),
        (
            "[`ICalExporter`](https://github.com/vacanza/holidays/blob/main/holidays/ical.py)",
            "[`ICalExporter`][holidays.ical.ICalExporter]",
        ),
        (
            "[`country_holidays`](https://github.com/vacanza/holidays/blob/dev/holidays/utils.py)",
            "[`country_holidays`][holidays.utils.country_holidays]",
        ),
        (
            "[`financial_holidays`](https://github.com/vacanza/holidays/blob/dev/holidays/utils.py)",
            "[`financial_holidays`][holidays.utils.financial_holidays]",
        ),
        (
            "[`generate`](https://github.com/vacanza/holidays/blob/main/holidays/ical.py)",
            "[`generate`][holidays.ical.ICalExporter.generate]",
        ),
        (
            "[`save_ics`](https://github.com/vacanza/holidays/blob/main/holidays/ical.py)",
            "[`save_ics`][holidays.ical.ICalExporter.save_ics]",
        ),
    ),
    "index.md": (
        (
            "[CONTRIBUTING.md](https://github.com/vacanza/holidays/blob/dev/CONTRIBUTING.md)",
            "[Contributing](contributing.md)",
        ),
        (
            "[`HolidayBase`](https://github.com/vacanza/holidays/blob/main/holidays/holiday_base.py)",
            "[`HolidayBase`][holidays.holiday_base.HolidayBase]",
        ),
    ),
}


def gfm_to_mkdocs(text: str) -> str:
    """Convert GitHub Flavored Markdown notes to ProperDocs admonitions."""
    lines = text.splitlines()
    out = []
    i = 0
    while i < len(lines):
        line = lines[i]
        if not (match := re.match(r"^>\s*\[!(\w+)\]$", line)):
            out.append(line)
            i += 1
            continue

        note_type = GFM_MKDOCS_MAPPING.get(match.group(1).lower(), "note")
        i += 1

        title = ""
        if i < len(lines):
            if match := re.match(r"^>\s*\*\*(.+?)\*\*", lines[i]):
                title = f' "{match.group(1)}"'
                i += 1

        note_body = []
        while i < len(lines):
            line = lines[i]
            if not line.startswith(">"):
                break

            content = line[1:]
            if content.startswith(" "):
                content = content[1:]
            note_body.append(content)
            i += 1

        out.append(f"!!! {note_type}{title}")
        out.extend(f"    {note_line}" if note_line else "" for note_line in note_body)

    return "\n".join(out)


def main():
    project_root = Path(__file__).parents[2]

    for target_path, source_path in (
        ("contributing.md", "CONTRIBUTING.md"),
        ("index.md", "README.md"),
        ("examples.md", "docs/examples.md"),
    ):
        with mkdocs_gen_files.open(target_path, "w", encoding="utf-8", newline="\n") as f:
            content = (project_root / source_path).read_text(encoding="utf-8")
            for old_str, new_str in FILES_REPLACE_MAPPING.get(target_path, []):
                content = content.replace(old_str, new_str)
            f.write(gfm_to_mkdocs(content))


if __name__ in {"__main__", "<run_path>"}:
    main()



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.seychelles import Seychelles
from tests.common import CommonCountryTests


class TestSeychelles(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Seychelles)

    def test_special_holidays(self):
        # Election Dates have its own separate checklists.
        self.assertHoliday(
            "2007-05-12",
            "2011-05-21",
            "2011-10-01",
            "2015-12-05",
            "2015-12-18",
            "2016-09-10",
            "2019-03-07",
            "2020-01-03",
            "2020-10-24",
            "2020-10-26",
        )

    def test_2009(self):
        # https://web.archive.org/web/20091208104016/https://www.cbs.sc/PublicHolidays.html
        self.assertHolidaysInYear(
            2009,
            ("2009-01-01", "New Year's Day"),
            ("2009-01-02", "New Year Holiday"),
            ("2009-04-10", "Good Friday"),
            ("2009-04-11", "Easter Saturday"),
            ("2009-05-01", "Labour Day"),
            ("2009-06-05", "Liberation Day"),
            ("2009-06-11", "The Fete Dieu"),
            ("2009-06-18", "National Day"),
            ("2009-06-29", "Independence Day"),
            ("2009-08-15", "Assumption Day"),
            ("2009-11-01", "All Saints Day"),
            ("2009-11-02", "All Saints Day (observed)"),
            ("2009-12-08", "The Feast of the Immaculate Conception"),
            ("2009-12-25", "Christmas Day"),
        )

    def test_2012(self):
        # https://web.archive.org/web/20121023221205/https://www.cbs.sc/PublicHolidays.html
        self.assertHolidaysInYear(
            2012,
            ("2012-01-01", "New Year's Day"),
            ("2012-01-02", "New Year Holiday"),
            ("2012-04-06", "Good Friday"),
            ("2012-04-07", "Easter Saturday"),
            ("2012-05-01", "Labour Day"),
            ("2012-06-05", "Liberation Day"),
            ("2012-06-07", "The Fete Dieu"),
            ("2012-06-18", "National Day"),
            ("2012-06-29", "Independence Day"),
            ("2012-08-15", "Assumption Day"),
            ("2012-11-01", "All Saints Day"),
            ("2012-12-08", "The Feast of the Immaculate Conception"),
            ("2012-12-25", "Christmas Day"),
        )

    def test_2019(self):
        # https://web.archive.org/web/20191029202210/http://cbs.sc/PublicHolidays.html
        self.assertHolidaysInYear(
            2019,
            ("2019-01-01", "New Year's Day"),
            ("2019-01-02", "New Year Holiday"),
            ("2019-03-07", "Funeral of the Former President France Albert René"),
            ("2019-04-19", "Good Friday"),
            ("2019-04-20", "Easter Saturday"),
            ("2019-04-22", "Easter Monday"),
            ("2019-05-01", "Labour Day"),
            ("2019-06-20", "The Fete Dieu"),
            ("2019-06-18", "Constitution Day"),
            ("2019-06-29", "Independence (National) Day"),
            ("2019-08-15", "Assumption Day"),
            ("2019-11-01", "All Saints Day"),
            ("2019-12-08", "The Feast of the Immaculate Conception"),
            ("2019-12-09", "The Feast of the Immaculate Conception (observed)"),
            ("2019-12-25", "Christmas Day"),
        )

    def test_2021(self):
        # https://web.archive.org/web/20211206090711/https://www.cbs.sc/PublicHolidays.html
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "New Year's Day"),
            ("2021-01-02", "New Year Holiday"),
            ("2021-04-02", "Good Friday"),
            ("2021-04-03", "Easter Saturday"),
            ("2021-04-05", "Easter Monday"),
            ("2021-05-01", "Labour Day"),
            ("2021-06-03", "The Fete Dieu"),
            ("2021-06-18", "Constitution Day"),
            ("2021-06-29", "Independence (National) Day"),
            ("2021-08-15", "Assumption Day"),
            ("2021-08-16", "Assumption Day (observed)"),
            ("2021-11-01", "All Saints Day"),
            ("2021-12-08", "The Feast of the Immaculate Conception"),
            ("2021-12-25", "Christmas Day"),
        )

    def test_2023(self):
        # https://web.archive.org/web/20230318041823/https://www.cbs.sc/PublicHolidays.html
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year Holiday"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-08", "Easter Saturday"),
            ("2023-04-10", "Easter Monday"),
            ("2023-05-01", "Labour Day"),
            ("2023-06-08", "The Fete Dieu"),
            ("2023-06-18", "Constitution Day"),
            ("2023-06-19", "Constitution Day (observed)"),
            ("2023-06-29", "Independence (National) Day"),
            ("2023-08-15", "Assumption Day"),
            ("2023-11-01", "All Saints Day"),
            ("2023-12-08", "The Feast of the Immaculate Conception"),
            ("2023-12-25", "Christmas Day"),
        )

    def test_l10n_default(self):
        # https://www.psb.gov.sc/public-holidays
        self.assertLocalizedHolidays(
            ("2024-01-01", "New Year's Day"),
            ("2024-01-02", "New Year Holiday"),
            ("2024-03-29", "Good Friday"),
            ("2024-03-30", "Easter Saturday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-01", "Labour Day"),
            ("2024-05-30", "The Fete Dieu"),
            ("2024-06-18", "Constitution Day"),
            ("2024-06-29", "Independence (National) Day"),
            ("2024-08-15", "Assumption Day"),
            ("2024-11-01", "All Saints Day"),
            ("2024-12-08", "The Feast of the Immaculate Conception"),
            ("2024-12-09", "The Feast of the Immaculate Conception (observed)"),
            ("2024-12-25", "Christmas Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-01-02", "New Year Holiday"),
            ("2024-03-29", "Good Friday"),
            ("2024-03-30", "Easter Saturday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-01", "Labor Day"),
            ("2024-05-30", "Corpus Christi"),
            ("2024-06-18", "Constitution Day"),
            ("2024-06-29", "Independence (National) Day"),
            ("2024-08-15", "Assumption Day"),
            ("2024-11-01", "All Saints' Day"),
            ("2024-12-08", "Immaculate Conception"),
            ("2024-12-09", "Immaculate Conception (observed)"),
            ("2024-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.svalbard_and_jan_mayen import SvalbardAndJanMayen
from tests.common import CommonCountryTests


class TestSvalbardAndJanMayen(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(SvalbardAndJanMayen)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Første nyttårsdag"),
            ("2022-04-14", "Skjærtorsdag"),
            ("2022-04-15", "Langfredag"),
            ("2022-04-17", "Første påskedag"),
            ("2022-04-18", "Andre påskedag"),
            ("2022-05-01", "Arbeidernes dag"),
            ("2022-05-17", "Grunnlovsdag"),
            ("2022-05-26", "Kristi himmelfartsdag"),
            ("2022-06-05", "Første pinsedag"),
            ("2022-06-06", "Andre pinsedag"),
            ("2022-12-25", "Første juledag"),
            ("2022-12-26", "Andre juledag"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-04-14", "Maundy Thursday"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-17", "Constitution Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-06-05", "Whit Sunday"),
            ("2022-06-06", "Whit Monday"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Second Day of Christmas"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-04-14", "Великий четвер"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-17", "Великдень"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-05-01", "День праці"),
            ("2022-05-17", "День Конституції"),
            ("2022-05-26", "Вознесіння Господнє"),
            ("2022-06-05", "Трійця"),
            ("2022-06-06", "День Святого Духа"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Другий день Різдва"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2022-01-01", "วันขึ้นปีใหม่"),
            ("2022-04-14", "วันพฤหัสศักดิ์สิทธิ์"),
            ("2022-04-15", "วันศุกร์ประเสริฐ"),
            ("2022-04-17", "วันอาทิตย์อีสเตอร์"),
            ("2022-04-18", "วันจันทร์อีสเตอร์"),
            ("2022-05-01", "วันแรงงาน"),
            ("2022-05-17", "วันรัฐธรรมนูญ"),
            ("2022-05-26", "วันสมโภชพระเยซูเจ้าเสด็จขึ้นสวรรค์"),
            ("2022-06-05", "วันสมโภชพระจิตเจ้า"),
            ("2022-06-06", "วันจันทร์หลังวันสมโภชพระจิตเจ้า"),
            ("2022-12-25", "วันคริสต์มาสวันแรก"),
            ("2022-12-26", "วันคริสต์มาสวันที่สอง"),
        )
